@@ -83,6 +83,16 @@ string padAndConvert(const wstring &iWstr, unsigned int iLength,
 string centerAndConvert(const wstring &iWstr, unsigned int iLength,
                         char c = ' ');
 
+/**
+ * Return the upper case version of the given string
+ */
+wstring toUpper(const wstring &iWstr);
+
+/**
+ * Return the lower case version of the given string
+ */
+wstring toLower(const wstring &iWstr);
+
 /**
  * Utility function to convert a char* buffer encoded in UTF-8 into a
  * wchar_t* string
@@ -101,13 +111,11 @@ unsigned int readFromUTF8(wchar_t *oString, unsigned int iWideSize,
  * Same as the other readFromUTF8 function, dealing with a wstring
  * instead of a wchar_t*. Note that it performs an additional copy
  * of the output string...
- * @param iBuffer: UTF-8 string to convert
- * @param iBufSize: available size in iBuffer
+ * @param iString: UTF-8 string to convert
  * @param iContext: free text used in case of exception
  * @return: the converted wide string
  */
-wstring readFromUTF8(const char *iBuffer, unsigned int iBufSize,
-                     const string &iContext);
+wstring readFromUTF8(const string &iString, const string &iContext);
 
 /**
  * Utility function to convert a wstring into an UTF-8 char* buffer
@@ -130,5 +138,19 @@ unsigned int writeInUTF8(const wstring &iWString, char *oBuffer,
  */
 string writeInUTF8(const wstring &iWString, const string &iContext);
 
+
+// Useful macros
+
+// Convert between wstring and string
+#define wfl(s) convertToWc(s)
+#define lfw(s) convertToMb(s)
+// Convert between wstring and utf-8 string
+#define ufw(s) writeInUTF8((s), "ufw")
+#define wfu(s) readFromUTF8((s), "wfu")
+// Convert between string and utf-8 string
+#define ufl(s) ufw(wfl(s))
+#define lfu(s) lfw(wfu(s))
+
+
 #endif
 
