@@ -20,20 +20,21 @@
 
 #include <string>
 #include <stack>
-#include <boost/spirit/core.hpp>
-#include <boost/spirit/utility/chset.hpp>
-#include <boost/spirit/tree/ast.hpp>
+#include <boost/spirit/include/classic_core.hpp>
+#include <boost/spirit/include/classic_chset.hpp>
+#include <boost/spirit/include/classic_ast.hpp>
 #ifdef DEBUG_RE
-#include <boost/spirit/tree/tree_to_xml.hpp>
+#include <boost/spirit/include/classic_tree_to_xml.hpp>
 #include <map>
 #include <iostream>
 #endif
 
 #include "dic.h"
 #include "header.h"
+#include "encoding.h"
 #include "regexp.h"
 
-using namespace boost::spirit;
+using namespace boost::spirit::classic;
 using namespace std;
 
 // A few typedefs to simplify things
@@ -54,9 +55,7 @@ struct RegexpGrammar : grammar<RegexpGrammar>
 
     RegexpGrammar(const wstring &letters)
     {
-        wstring lower = letters;
-        std::transform(lower.begin(), lower.end(), lower.begin(), towlower);
-        m_allLetters = letters + lower;
+        m_allLetters = letters + toLower(letters);
     }
 
     template <typename ScannerT>
@@ -138,8 +137,7 @@ void evaluate(const Header &iHeader, iter_t const& i, stack<Node*> &evalStack,
 
         wstring choiceLetters(i->value.begin(), i->value.end());
         // Make sure the letters are in upper case
-        std::transform(choiceLetters.begin(), choiceLetters.end(),
-                       choiceLetters.begin(), towupper);
+        choiceLetters = toUpper(choiceLetters);
         // The dictionary letters are already in upper case
         const wstring &letters = iHeader.getLetters();
         wstring::const_iterator itLetter;
