@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2009 Olivier Teulière
+ * Copyright (C) 2005-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,11 +18,12 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-#ifndef _AI_PERCENT_H_
-#define _AI_PERCENT_H_
+#ifndef AI_PERCENT_H_
+#define AI_PERCENT_H_
 
 #include "ai_player.h"
 #include "results.h"
+#include "logging.h"
 
 /**
  * This kind of AI is parameterized by a percentage p.
@@ -37,11 +38,14 @@
  */
 class AIPercent: public AIPlayer
 {
+    DEFINE_LOGGER();
 public:
     /// Constructor, taking the percentage (0.0 <= iPercent <= 1.0)
     AIPercent(float iPercent);
     virtual ~AIPercent();
 
+    float getPercent() const { return m_percent; }
+
     /**
      * This method does the actual computation. It will be called before any
      * of the following methods, so it must prepare everything for them.
@@ -52,6 +56,7 @@ public:
     virtual Move getMove() const;
 
 private:
+    float m_percent;
     /// Container for all the found solutions
     Results *m_results;
 };
