@@ -45,17 +45,23 @@ public:
     /// Replace a tile into the bag
     void replaceTile(const Tile &iTile);
 
-    /// Return how many tiles identical to iTile are available in the bag
-    unsigned int in(const Tile &iTile) const;
+    /// Count how many tiles identical to iTile are available in the bag
+    unsigned count(const Tile &iTile) const;
+
+    /**
+     * Return true if the bag contains the given letter.
+     * This is a shortcut for: count(iTile) != 0
+     */
+    bool contains(const Tile &iTile) const { return count(iTile); }
 
     /**
      * Return how many tiles/vowels/consonants are available
      * Warning: b.getNbVowels() + b.getNbConsonants() != b.getNbTiles(),
      * because of the jokers and the 'Y'.
      */
-    unsigned int getNbTiles() const  { return m_ntiles; }
-    unsigned int getNbVowels() const;
-    unsigned int getNbConsonants() const;
+    unsigned getNbTiles() const { return m_nbTiles; }
+    unsigned getNbVowels() const;
+    unsigned getNbConsonants() const;
 
     /**
      * Return a random available tile
@@ -75,14 +81,11 @@ public:
      */
     Tile selectRandomConsonant() const;
 
-    void operator=(const Bag &iOther);
+    Bag & operator=(const Bag &iOther);
 
     /// Convenience getter on the dictionary
     const Dictionary & getDic() const { return m_dic; }
 
-    /// Print on stderr all the letters of the bag (for debugging purposes)
-    void dumpAll() const;
-
 private:
     /// Dictionary
     const Dictionary &m_dic;
@@ -91,10 +94,10 @@ private:
     map<Tile, int> m_tilesMap;
 
     /// Total number of tiles in the bag
-    int m_ntiles;
+    unsigned m_nbTiles;
 
     /// Helper method, used by the various selectRandom*() methods
-    Tile selectRandomTile(unsigned int total,
+    Tile selectRandomTile(unsigned total,
                           bool onlyVowels, bool onlyConsonants) const;
 };
 
