@@ -30,6 +30,7 @@
 #include "logging.h"
 
 class GameParams;
+class BoardLayout;
 class Dictionary;
 class Rack;
 class Round;
@@ -40,7 +41,6 @@ using namespace std;
 #define BOARD_MIN 1
 #define BOARD_MAX 15
 #define BOARD_DIM 15
-#define BOARD_REALDIM (BOARD_DIM + 2)
 
 
 /**
@@ -55,6 +55,8 @@ class Board
 public:
     Board(const GameParams &iParams);
 
+    const BoardLayout & getLayout() const { return m_layout; }
+
     bool isJoker(int iRow, int iCol) const;
     bool isVacant(int iRow, int iCol) const;
 
@@ -63,7 +65,7 @@ public:
 
     void addRound(const Dictionary &iDic, const Round &iRound);
     void removeRound(const Dictionary &iDic, const Round &iRound);
-    int  checkRound(Round &iRound) const;
+    int  checkRound(Round &iRound, bool checkJunction = true) const;
 
     /**
      * Preview
@@ -76,12 +78,6 @@ public:
     void search(const Dictionary &iDic, const Rack &iRack, Results &oResults) const;
     void searchFirst(const Dictionary &iDic, const Rack &iRack, Results &oResults) const;
 
-    /**
-     *
-     */
-    static int GetWordMultiplier(int iRow, int iCol);
-    static int GetLetterMultiplier(int iRow, int iCol);
-
     /**
      * 
      */
@@ -92,6 +88,8 @@ private:
 
     const GameParams &m_params;
 
+    const BoardLayout &m_layout;
+
     Matrix<Tile> m_tilesRow;
     Matrix<Tile> m_tilesCol;
 
@@ -109,9 +107,6 @@ private:
     /// Flag indicating if the board is empty or if it has letters
     bool m_isEmpty;
 
-    static const int m_tileMultipliers[BOARD_REALDIM][BOARD_REALDIM];
-    static const int m_wordMultipliers[BOARD_REALDIM][BOARD_REALDIM];
-
     /**
      * board_cross.c
      */
@@ -121,7 +116,8 @@ private:
                       const Matrix<Cross> &iCrossMx,
                       const Matrix<int> &iPointsMx,
                       const Matrix<bool> &iJokerMx,
-                      Round &iRound) const;
+                      Round &iRound,
+                      bool checkJunction) const;
 #ifdef DEBUG
     void checkDouble();
 #endif
