@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 1999-2007 Antoine Fraboulet & Olivier Teulire
+ * Copyright (C) 1999-2012 Antoine Fraboulet & Olivier Teulire
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulire <ipkiss @@ gmail.com>
  *
@@ -19,13 +19,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-/**
- *  \file   board_cross.cpp
- *  \brief  Build cross information used to speed up search
- *  \author Antoine Fraboulet & Olivier Teulire
- *  \date   2005
- */
-
 #include <wctype.h>
 
 #include <dic.h>
@@ -41,13 +34,10 @@ static void Board_checkout_tile(const Dictionary &iDic,
                                 int& oPoints,
                                 int index)
 {
-    int i, left;
-    unsigned int node, succ;
-
     oPoints = 0;
 
     /* Points on the left part */
-    left = index;
+    int left = index;
     while (!iTiles[left - 1].isEmpty())
     {
         left--;
@@ -56,19 +46,18 @@ static void Board_checkout_tile(const Dictionary &iDic,
     }
 
     // FIXME: create temporary strings until the dictionary uses Tile objects
-    wchar_t leftTiles [BOARD_DIM + 1];
-    wchar_t rightTiles[BOARD_DIM + 1];
+    wstring leftTiles;
+    wstring rightTiles;
 
-    for (i = left; i < index; i++)
-        leftTiles[i - left] = towupper(iTiles[i].toChar());
-    leftTiles[index - left] = 0;
+    for (int i = left; i < index; i++)
+        leftTiles.push_back(towupper(iTiles[i].toChar()));
 
-    for (i = index + 1; !iTiles[i].isEmpty(); i++)
-        rightTiles[i - index - 1] = towupper(iTiles[i].toChar());
-    rightTiles[i - index - 1] = 0;
+    for (int i = index + 1; !iTiles[i].isEmpty(); i++)
+        rightTiles.push_back(towupper(iTiles[i].toChar()));
 
     /* Tiles that can be played */
-    node = iDic.charLookup(iDic.getRoot(), leftTiles);
+    unsigned int node, succ;
+    node = iDic.charLookup(iDic.getRoot(), leftTiles.c_str());
     if (node == 0)
     {
         oCross.setNone();
@@ -77,7 +66,7 @@ static void Board_checkout_tile(const Dictionary &iDic,
 
     for (succ = iDic.getSucc(node); succ; succ = iDic.getNext(succ))
     {
-        if (iDic.isEndOfWord(iDic.charLookup(succ, rightTiles)))
+        if (iDic.isEndOfWord(iDic.charLookup(succ, rightTiles.c_str())))
             oCross.insert(Tile(iDic.getChar(succ)));
         if (iDic.isLast(succ))
             break;
@@ -135,13 +124,3 @@ void Board::buildCross(const Dictionary &iDic)
     Board_check(iDic, m_tilesCol, m_jokerCol, m_crossRow, m_pointRow);
 }
 
-
-/****************************************************************/
-/****************************************************************/
-
-/// Local Variables:
-/// mode: c++
-/// mode: hs-minor
-/// c-basic-offset: 4
-/// indent-tabs-mode: nil
-/// End:
