@@ -25,6 +25,8 @@
 #include "matrix.h"
 
 class Dictionary;
+class GameParams;
+class Tile;
 class Rack;
 class Results;
 class Round;
@@ -35,6 +37,7 @@ class BoardSearch
 {
 public:
     BoardSearch(const Dictionary &iDic,
+                const GameParams &iParams,
                 const Matrix<Tile> &iTilesMx,
                 const Matrix<Cross> &iCrossMx,
                 const Matrix<int> &iPointsMx,
@@ -45,6 +48,7 @@ public:
 
 private:
     const Dictionary &m_dic;
+    const GameParams &m_params;
     const Matrix<Tile> &m_tilesMx;
     const Matrix<Cross> &m_crossMx;
     const Matrix<int> &m_pointsMx;
