@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2005-2007 Olivier Teulière & Antoine Fraboulet
+ * Copyright (C) 2005-2012 Olivier Teulière & Antoine Fraboulet
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *          Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *
@@ -19,11 +19,12 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-#ifndef _CROSS_H_
-#define _CROSS_H_
+#ifndef CROSS_H_
+#define CROSS_H_
 
 #include <set>
 #include "tile.h"
+#include "logging.h"
 
 using namespace std;
 
@@ -35,6 +36,7 @@ using namespace std;
 // TODO: implement using the bitset class
 class Cross
 {
+    DEFINE_LOGGER();
 public:
     Cross();
 
