@@ -33,11 +33,11 @@
 #include "pldrack.h"
 #include "results.h"
 #include "player.h"
-#include "player_event_cmd.h"
-#include "player_move_cmd.h"
-#include "player_rack_cmd.h"
-#include "game_move_cmd.h"
-#include "game_rack_cmd.h"
+#include "cmd/player_event_cmd.h"
+#include "cmd/player_move_cmd.h"
+#include "cmd/player_rack_cmd.h"
+#include "cmd/game_move_cmd.h"
+#include "cmd/game_rack_cmd.h"
 #include "ai_player.h"
 #include "settings.h"
 #include "turn_data.h"
@@ -48,8 +48,8 @@
 INIT_LOGGER(game, FreeGame);
 
 
-FreeGame::FreeGame(const GameParams &iParams)
-    : Game(iParams), m_finished(false)
+FreeGame::FreeGame(const GameParams &iParams, const Game *iMasterGame)
+    : Game(iParams, iMasterGame), m_finished(false)
 {
 }
 
@@ -57,32 +57,16 @@ FreeGame::FreeGame(const GameParams &iParams)
 int FreeGame::play(const wstring &iCoord, const wstring &iWord)
 {
     // Perform all the validity checks, and try to fill a round
-    Round round;
-    int res = checkPlayedWord(iCoord, iWord, round);
+    Move move;
+    int res = checkPlayedWord(iCoord, iWord, move);
     if (res != 0 && Settings::Instance().getBool("freegame.reject-invalid"))
     {
         return res;
     }
 
     // If we reach this point, either the move is valid and we can use the
-    // "round" variable, or it is invalid but played nevertheless
-    if (res == 0)
-    {
-        Move move(round);
-
-        // Update the rack and the score of the current player
-        recordPlayerMove(move, *m_players[m_currPlayer]);
-    }
-    else
-    {
-        // Convert the invalid word for display
-        const wdstring &dispWord = getDic().convertToDisplay(iWord);
-
-        Move move(dispWord, iCoord);
-
-        // Record the invalid move of the player
-        recordPlayerMove(move, *m_players[m_currPlayer]);
-    }
+    // "move" variable, or it is invalid but played nevertheless
+    recordPlayerMove(move, *m_players[m_currPlayer]);
 
     // Next turn
     endTurn();
@@ -126,11 +110,11 @@ void FreeGame::start()
     ASSERT(getNPlayers(), "Cannot start a game without any player");
 
     // Set the initial racks of the players
-    for (unsigned int i = 0; i < getNPlayers(); i++)
+    BOOST_FOREACH(Player *player, m_players)
     {
         const PlayedRack &newRack =
-            helperSetRackRandom(getPlayer(i).getCurrentRack(), false, RACK_NEW);
-        Command *pCmd = new PlayerRackCmd(*m_players[i], newRack);
+            helperSetRackRandom(player->getCurrentRack(), false, RACK_NEW);
+        Command *pCmd = new PlayerRackCmd(*player, newRack);
         accessNavigation().addAndExecute(pCmd);
     }
 
@@ -169,7 +153,7 @@ int FreeGame::endTurn()
 
     const Move &move = getCurrentPlayer().getLastMove();
     // Update the game
-    Command *pCmd = new GameMoveCmd(*this, move, m_currPlayer);
+    Command *pCmd = new GameMoveCmd(*this, move);
     accessNavigation().addAndExecute(pCmd);
 
     // Complete the rack for the player that just played
@@ -286,7 +270,7 @@ int FreeGame::checkPass(const Player &iPlayer,
     BOOST_FOREACH(wchar_t wch, iToChange)
     {
         // Remove the letter from the rack
-        if (!rack.in(Tile(wch)))
+        if (!rack.contains(Tile(wch)))
         {
             return 2;
         }
