@@ -43,14 +43,18 @@
 #include "encoding.h"
 #include "game_exception.h"
 #include "turn.h"
+#include "cmd/player_rack_cmd.h"
+#include "cmd/player_move_cmd.h"
+#include "cmd/game_rack_cmd.h"
 
 #include "debug.h"
 
 INIT_LOGGER(game, Game);
 
 
-Game::Game(const GameParams &iParams):
-    m_params(iParams), m_board(m_params), m_bag(iParams.getDic())
+Game::Game(const GameParams &iParams, const Game *iMasterGame):
+    m_params(iParams), m_masterGame(iMasterGame),
+    m_board(m_params), m_bag(iParams.getDic())
 {
     m_points = 0;
     m_currPlayer = 0;
@@ -63,6 +67,7 @@ Game::~Game()
     {
         delete p;
     }
+    delete m_masterGame;
 }
 
 
@@ -89,6 +94,22 @@ void Game::shuffleRack()
 }
 
 
+void Game::reorderRack(const PlayedRack &iNewRack)
+{
+    LOG_DEBUG("Reordering rack for player " << currPlayer() <<
+              " (newRack=" << lfw(iNewRack.toString()) << ")");
+    const PlayedRack &pld = getCurrentPlayer().getCurrentRack();
+
+    // Make sure the new rack uses the same letters
+    ASSERT(pld.getRack() == iNewRack.getRack(),
+           "The old and new racks have different letters" <<
+           "(old=" << lfw(pld.toString()) << 
+           " new=" << lfw(iNewRack.toString()) << ")");
+
+    m_players[currPlayer()]->setCurrentRack(iNewRack);
+}
+
+
 void Game::realBag(Bag &ioBag) const
 {
     // Copy the bag
@@ -182,9 +203,56 @@ bool Game::canDrawRack(const PlayedRack &iPld, bool iCheck, int *reason) const
 }
 
 
+PlayedRack Game::getRackFromMasterGame() const
+{
+    ASSERT(hasMasterGame(), "No master game defined");
+
+    // End the game when we reach the end of the master game,
+    // even if it is still possible to draw a rack
+    const unsigned currTurn = getNavigation().getCurrTurn();
+    if (currTurn >= m_masterGame->getHistory().getSize())
+        throw EndGameException(_("No more turn in the master game"));
+
+    const TurnData &turnData = m_masterGame->getHistory().getTurn(currTurn);
+    const PlayedRack &pldRack = turnData.getPlayedRack();
+    LOG_INFO("Using rack from master game: " << lfw(pldRack.toString()));
+
+    // Sanity check
+    ASSERT(rackInBag(pldRack.getRack(), m_bag),
+           "Cannot draw same rack as in the master game");
+
+    return pldRack;
+}
+
+
+Move Game::getMoveFromMasterGame() const
+{
+    ASSERT(hasMasterGame(), "No master game defined");
+
+    const unsigned currTurn = getNavigation().getCurrTurn();
+    // Should never happen (already checked in getRackFromMasterGame())
+    ASSERT(currTurn < m_masterGame->getHistory().getSize(),
+           "Not enough turns in the master game");
+
+    const TurnData &turnData = m_masterGame->getHistory().getTurn(currTurn);
+    const Move &move = turnData.getMove();
+
+    // If the move is not valid, it means we reached the end
+    // of the master game. In this case, also end the current game.
+    if (!move.isValid())
+        throw EndGameException(_("No move defined for this turn in the master game"));
+
+    return move;
+}
+
+
 PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
                                      bool iCheck, set_rack_mode mode) const
 {
+    // If a master game is defined, use it to retrieve the rack
+    if (hasMasterGame())
+        return getRackFromMasterGame();
+
     int reason = 0;
     if (!canDrawRack(iPld, iCheck, &reason))
     {
@@ -256,9 +324,11 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
     vector<Tile> tiles;
     pld.getOldTiles(tiles);
     // The rack is already complete, there is nothing to do
-    // TODO: add a log here
     if (tiles.size() >= RACK_SIZE)
+    {
+        LOG_DEBUG("Rack already complete, nothing to do");
         return iPld;
+    }
 
     bool jokerAdded = false;
     // Are we dealing with a normal game or a joker game?
@@ -277,7 +347,7 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
         }
 
         // 2) If there was no joker, we add one if possible
-        if (!jokerFound && bag.in(Tile::Joker()))
+        if (!jokerFound && bag.contains(Tile::Joker()))
         {
             jokerAdded = true;
             pld.addNew(Tile::Joker());
@@ -285,7 +355,7 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
         }
 
         // 3) Remove all the jokers from the bag, to avoid taking another one
-        while (bag.in(Tile::Joker()))
+        while (bag.contains(Tile::Joker()))
         {
             bag.takeTile(Tile::Joker());
         }
@@ -381,9 +451,9 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
     {
         const Rack &rack = pld.getRack();
 
-        BestResults res;
+        MasterResults res(getBag());
         res.search(getDic(), getBoard(), rack,  getHistory().beforeFirstRound());
-        if (res.size())
+        if (!res.isEmpty())
         {
             PlayedRack pldCopy = pld;
 
@@ -393,37 +463,47 @@ PlayedRack Game::helperSetRackRandom(const PlayedRack &iPld,
                       << lfw(pld.toString()) << " (best word: "
                       << lfw(bestRound.getWord()) << ")");
 
-            // Identify the joker
+            // Identify the tile we should use to replace the joker
+            Tile replacingTile;
+            bool jokerUsed = false;
             for (unsigned int i = 0; i < bestRound.getWordLen(); ++i)
             {
                 if (bestRound.isJoker(i) && bestRound.isPlayedFromRack(i))
                 {
                     const Tile &jokerTile = bestRound.getTile(i);
-                    const Tile &replacingTile = jokerTile.toUpper();
-                    LOG_DEBUG("helperSetRackRandom(): replacing Joker with "
-                              << lfw(replacingTile.toChar()));
-
-                    // If the bag does not contain this letter anymore,
-                    // simply keep the joker in the rack.
-                    if (bag.in(replacingTile))
-                    {
-                        // The bag contains the replacing letter
-                        // We need to swap the joker (it is necessarily in the
-                        // new tiles, because jokerAdded is true)
-                        Rack tmpRack = pld.getNew();
-                        ASSERT(tmpRack.in(Tile::Joker()),
-                               "No joker found in the new tiles!");
-                        tmpRack.remove(Tile::Joker());
-                        tmpRack.add(replacingTile);
-                        pld.setNew(tmpRack);
-
-                        // Make sure the invariant is still correct, otherwise we keep the joker
-                        if (!pld.checkRack(min, min))
-                            pld = pldCopy;
-                    }
+                    replacingTile = jokerTile.toUpper();
+                    jokerUsed = true;
                     break;
                 }
             }
+            if (!jokerUsed)
+            {
+                // The joker was not needed for the top. Replace it with a
+                // randomly selected tile
+                LOG_DEBUG("helperSetRackRandom(): joker not needed for the top");
+                replacingTile = bag.selectRandom();
+            }
+
+            LOG_DEBUG("helperSetRackRandom(): replacing Joker with "
+                      << lfw(replacingTile.toChar()));
+
+            // If the bag does not contain the letter anymore,
+            // simply keep the joker in the rack.
+            if (bag.contains(replacingTile))
+            {
+                // The bag contains the replacing letter
+                // We need to swap the joker (it is necessarily in the
+                // new tiles, because jokerAdded is true)
+                Rack tmpRack = pld.getNew();
+                ASSERT(tmpRack.contains(Tile::Joker()), "No joker found in the new tiles");
+                tmpRack.remove(Tile::Joker());
+                tmpRack.add(replacingTile);
+                pld.setNew(tmpRack);
+
+                // Make sure the invariant is still correct, otherwise we keep the joker
+                if (!pld.checkRack(min, min))
+                    pld = pldCopy;
+            }
         }
     }
 
@@ -443,7 +523,7 @@ bool Game::rackInBag(const Rack &iRack, const Bag &iBag) const
 {
     BOOST_FOREACH(const Tile &t, getDic().getAllTiles())
     {
-        if (iRack.in(t) > iBag.in(t))
+        if (iRack.count(t) > iBag.count(t))
             return false;
     }
     return true;
@@ -526,23 +606,28 @@ void Game::nextPlayer()
 
 int Game::checkPlayedWord(const wstring &iCoord,
                           const wstring &iWord,
-                          Round &oRound, bool checkRack) const
+                          Move &oMove, bool checkRack,
+                          bool checkWordAndJunction) const
 {
     ASSERT(getNPlayers() != 0, "Expected at least one player");
 
+    // Assume that the move is invalid by default
+    const wdstring &dispWord = getDic().convertToDisplay(iWord);
+    oMove = Move(dispWord, iCoord);
+
     if (!getDic().validateLetters(iWord))
         return 1;
 
     // Init the round with the given coordinates
-    oRound = Round();
-    oRound.accessCoord().setFromString(iCoord);
-    if (!oRound.getCoord().isValid())
+    Round round;
+    round.accessCoord().setFromString(iCoord);
+    if (!round.getCoord().isValid())
     {
         return 2;
     }
 
     // Check the existence of the word
-    if (!getDic().searchWord(iWord))
+    if (checkWordAndJunction && !getDic().searchWord(iWord))
     {
         return 3;
     }
@@ -556,23 +641,24 @@ int Game::checkPlayedWord(const wstring &iCoord,
     {
         tiles.push_back(Tile(iWord[i]));
     }
-    oRound.setWord(tiles);
+    round.setWord(tiles);
 
     // Check the word position, compute its points,
     // and specify the origin of each letter (board or rack)
-    int res = m_board.checkRound(oRound);
+    int res = m_board.checkRound(round, checkWordAndJunction);
     if (res != 0)
         return res + 4;
     // In duplicate mode, the first word must be horizontal
-    if (m_board.isVacant(8, 8) &&
+    if (checkWordAndJunction && m_board.isVacant(8, 8) &&
         (getMode() == GameParams::kDUPLICATE ||
-         getMode() == GameParams::kARBITRATION))
+         getMode() == GameParams::kARBITRATION ||
+         getMode() == GameParams::kTOPPING))
     {
-        if (oRound.getCoord().getDir() == Coord::VERTICAL)
+        if (round.getCoord().getDir() == Coord::VERTICAL)
             return 10;
     }
 
-    if (checkRack)
+    if (checkWordAndJunction && checkRack)
     {
         // Check that the word can be formed with the tiles in the rack:
         // we first create a copy of the rack, then we remove the tiles
@@ -581,16 +667,16 @@ int Game::checkPlayedWord(const wstring &iCoord,
         Rack rack = player->getCurrentRack().getRack();
 
         Tile t;
-        for (unsigned int i = 0; i < oRound.getWordLen(); i++)
+        for (unsigned int i = 0; i < round.getWordLen(); i++)
         {
-            if (oRound.isPlayedFromRack(i))
+            if (round.isPlayedFromRack(i))
             {
-                if (oRound.isJoker(i))
+                if (round.isJoker(i))
                     t = Tile::Joker();
                 else
-                    t = oRound.getTile(i);
+                    t = round.getTile(i);
 
-                if (!rack.in(t))
+                if (!rack.contains(t))
                 {
                     return 4;
                 }
@@ -599,10 +685,42 @@ int Game::checkPlayedWord(const wstring &iCoord,
         }
     }
 
+    // The move is valid
+    oMove = Move(round);
+
     return 0;
 }
 
 
+void Game::setGameAndPlayersRack(const PlayedRack &iRack, bool iWithNoMove)
+{
+    // Set the game rack
+    Command *pCmd = new GameRackCmd(*this, iRack);
+    accessNavigation().addAndExecute(pCmd);
+    LOG_INFO("Setting players rack to '" + lfw(iRack.toString()) + "'");
+    // All the players have the same rack
+    BOOST_FOREACH(Player *player, m_players)
+    {
+        Command *pCmd = new PlayerRackCmd(*player, iRack);
+        accessNavigation().addAndExecute(pCmd);
+    }
+
+    if (iWithNoMove)
+    {
+        // Assign a "no move" pseudo-move to all the players.
+        // This avoids the need to distinguish between "has not played yet"
+        // and "has played with no move" in duplicate and arbitration modes.
+        // This is also practical to know at which turn the warnings, penalties
+        // and solos should be assigned.
+        BOOST_FOREACH(Player *player, m_players)
+        {
+            Command *pCmd = new PlayerMoveCmd(*player, Move());
+            accessNavigation().addAndExecute(pCmd);
+        }
+    }
+}
+
+
 Game::CurrentPlayerCmd::CurrentPlayerCmd(Game &ioGame,
                              unsigned int iPlayerId)
     : m_game(ioGame), m_newPlayerId(iPlayerId), m_oldPlayerId(0)
