@@ -51,7 +51,6 @@ class Game
 {
     DEFINE_LOGGER();
 public:
-    Game(const GameParams &iParams);
     virtual ~Game();
 
     /***************
@@ -67,6 +66,8 @@ public:
     /// Get the game characteristics
     const GameParams & getParams() const { return m_params; }
 
+    bool hasMasterGame() const { return m_masterGame != NULL; }
+
     /**
      * Get the dictionary associated with the game.
      * You should never create a new dictionary object while a Game
@@ -151,9 +152,25 @@ public:
      */
     virtual int play(const wstring &iCoord, const wstring &iWord) = 0;
 
-    /// Shuffle the rack of the current player
+    /**
+     * Shuffle the rack of the current player.
+     *
+     * Note that this is persistent only until the player gets a new rack.
+     * The shuffling will be lost by navigating in the game history, and
+     * will not be saved in savegames.
+     */
     void shuffleRack();
 
+    /**
+     * Change the rack of the current player.
+     * The letters must be the same as in the existing rack.
+     *
+     * Note that this is persistent only until the player gets a new rack.
+     * The reordering will be lost by navigating in the game history, and
+     * will not be saved in savegames.
+     */
+    void reorderRack(const PlayedRack &iNewRack);
+
     /// Return true if the player has played for the current turn
     // XXX: not very nice API, should be a player property...
     virtual bool hasPlayed(unsigned int player) const { return player != currPlayer(); }
@@ -171,7 +188,7 @@ public:
 
     /**
      * This function checks whether it is legal to play the given word at the
-     * given coordinates. If so, the function fills a Round object, also given
+     * given coordinates. If so, the function fills a Move object, also given
      * as a parameter.
      * Possible return values: same as the play() method
      * If checkRack is false, the return value 4 is impossible to get
@@ -179,13 +196,17 @@ public:
      */
     int checkPlayedWord(const wstring &iCoord,
                         const wstring &iWord,
-                        Round &oRound,
-                        bool checkRack = true) const;
+                        Move &oMove,
+                        bool checkRack = true,
+                        bool checkWordAndJunction = true) const;
 
 private:
     /// Game characteristics
     GameParams m_params;
 
+    /// Master game (can be NULL)
+    const Game *m_masterGame;
+
     /**
      * History of the game.
      */
@@ -195,6 +216,7 @@ private:
 
     int m_points;
 
+
     /// Change the player who is supposed to play
     void setCurrentPlayer(unsigned int iPlayerId) { m_currPlayer = iPlayerId; }
 
@@ -230,6 +252,13 @@ protected:
     /// Bag
     Bag m_bag;
 
+    /**
+     * Protected constructor.
+     * The iMasterGame parameter is optional (i.e. it can be NULL).
+     * This game takes ownership of the master game, if one is provided.
+     */
+    Game(const GameParams &iParams, const Game *iMasterGame);
+
     /*********************************************************
      * Helper functions
      *********************************************************/
@@ -277,6 +306,26 @@ protected:
      */
     PlayedRack helperSetRackManual(bool iCheck, const wstring &iLetters) const;
 
+    /**
+     * Helper method to retrieve the rack for the current turn
+     * in the master game. Should only be called when hasMasterGame()
+     * returns true.
+     */
+    PlayedRack getRackFromMasterGame() const;
+
+    /**
+     * Helper method to retrieve the move played for the current turn
+     * in the master game. Should only be called when hasMasterGame()
+     * returns true.
+     */
+    Move getMoveFromMasterGame() const;
+
+    /**
+     * Helper function to set the game rack and the players rack at the same time.
+     * Shouldn't be used in free game mode.
+     */
+    void setGameAndPlayersRack(const PlayedRack &iRack, bool iWithNoMove);
+
     void nextPlayer();
 
     /**
