@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2007 Olivier Teulière
+ * Copyright (C) 2007-2010 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,24 +21,19 @@
 #ifndef GAME_EXCEPTION_H_
 #define GAME_EXCEPTION_H_
 
-#include <exception>
 #include <string>
+#include "base_exception.h"
 
 
 /**
  * Exception class for the Game library.
- * It simply inherits from the standard exception and overrides
+ * It simply inherits from the base exception and overrides
  * its what() method.
  */
-class GameException: public std::exception
+class GameException: public BaseException
 {
     public:
         GameException(const std::string &iMessage);
-        ~GameException() throw() {}
-        virtual const char *what() const throw();
-
-    private:
-        std::string m_message;
 };
 
 
