@@ -50,7 +50,7 @@ class Move
         /**
          * Default constructor, corresponding to no move
          */
-        Move();
+        Move(int iPoints = 0);
 
         /**
          * Constructor taking a (valid) round
@@ -121,6 +121,9 @@ class Move
         /// To help debugging
         wstring toString() const;
 
+        /// Equality operator
+        bool operator==(const Move &iOther) const;
+
     private:
 
         enum Type
