@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2004-2007 Olivier Teulière & Antoine Fraboulet
+ * Copyright (C) 2004-2012 Olivier Teulière & Antoine Fraboulet
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *          Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *
@@ -19,17 +19,18 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-#ifndef _PLAYER_H_
-#define _PLAYER_H_
+#ifndef PLAYER_H_
+#define PLAYER_H_
 
 #include <vector>
 #include <string>
 #include "pldrack.h"
 #include "history.h"
+#include "logging.h"
 
 using std::wstring;
 
-class Turn;
+class TurnData;
 class Rack;
 
 
@@ -39,6 +40,7 @@ class Rack;
  */
 class Player
 {
+    DEFINE_LOGGER();
 public:
     explicit Player();
     virtual ~Player() {}
@@ -51,9 +53,13 @@ public:
     /// Set the name of the player
     void setName(const wstring &iName) { m_name = iName; }
 
+    /// Accessors for the table number
+    unsigned getTableNb() const { return m_tableNb; }
+    void setTableNb(unsigned tableNb) { m_tableNb = tableNb; }
+
     /// ID handling
     unsigned int getId() const { return m_id; }
-    void setId(unsigned int iId) { m_id = iId; }
+    void setId(unsigned int iId);
 
     /**************************
      * General getters
@@ -73,13 +79,26 @@ public:
     /// Remove last turn
     void removeLastTurn();
 
-    /**************************
-     * Accessors for the score of the player
-     **************************/
-    // Add (or remove, if iPoints is negative) points to the score
-    // of the player
-    void addPoints(int iPoints) { m_score += iPoints; }
-    int  getPoints() const      { return m_score; }
+    /// Return the total number of warnings received
+    unsigned getWarningsNb() const;
+
+    /// Return the total number of points received for moves alone
+    int getMovePoints() const;
+
+    /**
+     * Return the total number of penalties received
+     * (including the penalties due to warnings)
+     */
+    int getPenaltyPoints() const;
+
+    /// Return the total number of solo points received
+    int getSoloPoints() const;
+
+    /// Return the total number of end game points
+    int getEndGamePoints() const;
+
+    /// Return the total score of the player, including solos, penalties, ...
+    int getTotalScore() const;
 
     wstring toString() const;
 
@@ -87,12 +106,12 @@ private:
     /// ID of the player
     unsigned int m_id;
 
-    /// Score of the player
-    int m_score;
-
     /// Name of the player
     wstring m_name;
 
+    // Table number (optional: set to 0 if not used)
+    unsigned m_tableNb;
+
     /// History of the racks and rounds for the player
     History m_history;
 };
