@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2002-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 2002-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -26,6 +26,9 @@
 #include "rack.h"
 
 
+INIT_LOGGER(game, PlayedRack);
+
+
 PlayedRack::PlayedRack()
     : m_reject(false)
 {
@@ -80,33 +83,36 @@ void PlayedRack::resetNew()
 }
 
 
-void PlayedRack::getOld(Rack &oRack) const
+Rack PlayedRack::getOld() const
 {
-    oRack.clear();
+    Rack rack;
     BOOST_FOREACH(const Tile &tile, m_oldTiles)
     {
-        oRack.add(tile);
+        rack.add(tile);
     }
+    return rack;
 }
 
 
-void PlayedRack::getNew(Rack &oRack) const
+Rack PlayedRack::getNew() const
 {
-    oRack.clear();
+    Rack rack;
     BOOST_FOREACH(const Tile &tile, m_newTiles)
     {
-        oRack.add(tile);
+        rack.add(tile);
     }
+    return rack;
 }
 
 
-void PlayedRack::getRack(Rack &oRack) const
+Rack PlayedRack::getRack() const
 {
-    getOld(oRack);
+    Rack rack = getOld();
     BOOST_FOREACH(const Tile &tile, m_newTiles)
     {
-        oRack.add(tile);
+        rack.add(tile);
     }
+    return rack;
 }
 
 
