@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2002-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 2002-2012 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -19,12 +19,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-#ifndef _PLAYEDRACK_H_
-#define _PLAYEDRACK_H_
+#ifndef PLAYEDRACK_H_
+#define PLAYEDRACK_H_
 
 #include <vector>
 #include <string>
 #include "tile.h"
+#include "logging.h"
 
 class Rack;
 
@@ -39,15 +40,16 @@ using namespace std;
  */
 class PlayedRack
 {
+    DEFINE_LOGGER();
 public:
     PlayedRack();
 
     void reset();
     void resetNew();
 
-    void getOld(Rack &oRack) const;
-    void getNew(Rack &oRack) const;
-    void getRack(Rack &oRack) const;
+    Rack getOld() const;
+    Rack getNew() const;
+    Rack getRack() const;
 
     void setOld(const Rack &iRack);
     void setNew(const Rack &iRack);
