@@ -25,6 +25,7 @@
 #include "duplicate.h"
 #include "arbitration.h"
 #include "freegame.h"
+#include "topping.h"
 #include "game_factory.h"
 #include "game_exception.h"
 #include "xml_writer.h"
@@ -52,6 +53,8 @@ PublicGame::GameMode PublicGame::getMode() const
         return kDUPLICATE;
     else if (dynamic_cast<FreeGame*>(&m_game))
         return kFREEGAME;
+    else if (dynamic_cast<Topping*>(&m_game))
+        return kTOPPING;
     else
         return kTRAINING;
 }
@@ -63,6 +66,12 @@ const GameParams & PublicGame::getParams() const
 }
 
 
+bool PublicGame::hasMasterGame() const
+{
+    return m_game.hasMasterGame();
+}
+
+
 const Dictionary & PublicGame::getDic() const
 {
     return m_game.getDic();
@@ -159,13 +168,19 @@ int PublicGame::play(const wstring &iWord, const wstring &iCoord)
 }
 
 
+int PublicGame::checkPlayedWord(const wstring &iWord, const wstring &iCoord, Move &oMove) const
+{
+    return m_game.checkPlayedWord(iCoord, iWord, oMove, true, false);
+}
+
+
 int PublicGame::computePoints(const wstring &iWord, const wstring &iCoord) const
 {
-    Round round;
-    int res = m_game.checkPlayedWord(iCoord, iWord, round);
+    Move move;
+    int res = m_game.checkPlayedWord(iCoord, iWord, move);
     if (res > 0)
         return -res;
-    return round.getPoints();
+    return move.getScore();
 }
 
 
@@ -175,6 +190,12 @@ void PublicGame::shuffleRack()
 }
 
 
+void PublicGame::reorderRack(const PlayedRack &iRack)
+{
+    m_game.reorderRack(iRack);
+}
+
+
 void PublicGame::setTestRound(const Round &iRound)
 {
     m_game.accessBoard().testRound(iRound);
@@ -235,6 +256,37 @@ void PublicGame::trainingSetRackManual(bool iCheck, const wstring &iLetters)
 
 /***************************/
 
+void PublicGame::toppingPlay(const wstring &iWord, const wstring &iCoord, int iElapsed)
+{
+    getTypedGame<Topping>(m_game).tryWord(iWord, iCoord, iElapsed);
+}
+
+
+void PublicGame::toppingTimeOut(int iElapsed)
+{
+    getTypedGame<Topping>(m_game).turnTimeOut(iElapsed);
+}
+
+
+void PublicGame::toppingAddPenalty(int iPenalty)
+{
+    getTypedGame<Topping>(m_game).addPenalty(iPenalty);
+}
+
+
+vector<Move> PublicGame::toppingGetTriedMoves() const
+{
+    return getTypedGame<Topping>(m_game).getTriedMoves();
+}
+
+
+Move PublicGame::toppingGetTopMove() const
+{
+    return getTypedGame<Topping>(m_game).getTopMove();
+}
+
+/***************************/
+
 void PublicGame::duplicateSetPlayer(unsigned int p)
 {
     getTypedGame<Duplicate>(m_game).setPlayer(p);
