@@ -21,6 +21,7 @@
 #ifndef PUBLIC_GAME_H_
 #define PUBLIC_GAME_H_
 
+#include <vector>
 #include <string>
 
 class GameParams;
@@ -71,6 +72,7 @@ public:
         kFREEGAME,
         kDUPLICATE,
         kARBITRATION,
+        kTOPPING,
     };
     GameMode getMode() const;
 
@@ -81,6 +83,8 @@ public:
     /// Get the game characteristics
     const GameParams & getParams() const;
 
+    bool hasMasterGame() const;
+
     /**
      * Get the dictionary associated with the game.
      * You should never create a new dictionary object while a Game
@@ -158,6 +162,11 @@ public:
      */
     int play(const wstring &iWord, const wstring &iCoord);
 
+    // TODO: doc
+    // Ignores the word validity and connexion with the rest
+    int checkPlayedWord(const wstring &iWord, const wstring &iCoord,
+                        Move &oMove) const;
+
     /**
      * Compute the points for playing the word iWord at coordinates iCoord.
      * A negative return value indicates an error (same codes as for the
@@ -168,6 +177,12 @@ public:
     /// Shuffle the rack of the current player
     void shuffleRack();
 
+    /**
+     * Reorder the rack of the given player.
+     * An assertion is there to check that the letters are the same.
+     */
+    void reorderRack(const PlayedRack &iRack);
+
     /**
      * Place a temporary word on the board for preview purposes.
      * It is up to the caller to provide a Round
@@ -202,6 +217,23 @@ public:
 
     void trainingSetRackManual(bool iCheck, const wstring &iLetters);
 
+
+    /***************
+     * Topping games
+     * These methods throw an exception if the current game is not in
+     * the Topping mode.
+     ***************/
+
+    void toppingPlay(const wstring &iWord, const wstring &iCoord, int iElapsed);
+
+    void toppingTimeOut(int iElapsed);
+
+    void toppingAddPenalty(int iPenalty);
+
+    vector<Move> toppingGetTriedMoves() const;
+
+    Move toppingGetTopMove() const;
+
     /***************
      * Duplicate games
      * These methods throw an exception if the current game is not in
