@@ -41,21 +41,25 @@ class Rack
 public:
     Rack();
 
-    unsigned int getNbTiles() const      { return m_ntiles; }
+    unsigned getNbTiles() const      { return m_nbTiles; }
     bool isEmpty() const        { return getNbTiles() == 0; }
 
-    unsigned int in(const Tile &t) const { return m_tiles[t.toCode()]; }
-    void add(const Tile &t)     { m_tiles[t.toCode()]++; m_ntiles++; }
+    unsigned count(const Tile &t) const { return m_tiles[t.toCode()]; }
+    bool contains(const Tile &t) const { return count(t); }
+
+    void add(const Tile &t)     { m_tiles[t.toCode()]++; m_nbTiles++; }
     void remove(const Tile &t);
     void clear();
     void getTiles(vector<Tile> &oTiles) const;
 
     wstring toString() const;
 
+    bool operator==(const Rack &iOther) const;
+
 private:
     /// Vector indexed by tile codes, containing the number of tiles
     vector<unsigned int> m_tiles;
-    unsigned int m_ntiles;
+    unsigned m_nbTiles;
 };
 
 #endif
