@@ -233,8 +233,12 @@ Settings::Settings()
     // If true, a random rack is defined, otherwise the rack is left untouched
     arbitration.add("fill-rack", Setting::TypeBoolean) = true;
 
-    // Number of search results kept in a search
-    arbitration.add("search-limit", Setting::TypeInt) = 100;
+    // If true, solos are automatically given when appropriate
+    // If false, the arbitrator has full control (but must do everything manually)
+    arbitration.add("solo-auto", Setting::TypeBoolean) = true;
+    // Minimum number of players in a duplicate game needed to apply a "solo" bonus
+    // (16 is the ODS value)
+    arbitration.add("solo-players", Setting::TypeInt) = 16;
 
     // Number of points granted for a solo (10 is the ODS value)
     arbitration.add("solo-value", Setting::TypeInt) = 10;
@@ -245,6 +249,19 @@ Settings::Settings()
     // Maximum number of warnings before getting penalties
     arbitration.add("warnings-limit", Setting::TypeInt) = 3;
 
+    // Number of search results kept in a search
+    arbitration.add("search-limit", Setting::TypeInt) = 100;
+
+    // ============== Topping mode options ==============
+    Setting &topping = m_conf->getRoot().add("topping", Setting::TypeGroup);
+
+    // If true, a score penalty equal to the number of elapsed seconds
+    // is given to the player at each turn
+    topping.add("elapsed-penalty", Setting::TypeBoolean) = true;
+
+    // Additional penalty points given to the player when the timer expires
+    topping.add("timeout-penalty", Setting::TypeInt) = 60;
+
     // Try to read the values from the configuration file
     try
     {
@@ -260,13 +277,18 @@ Settings::Settings()
         copySetting<bool>(tmpConf, *m_conf, "freegame.reject-invalid");
         copySetting<bool>(tmpConf, *m_conf, "arbitration.fill-rack");
         copySetting<int>(tmpConf, *m_conf, "arbitration.search-limit");
+        copySetting<bool>(tmpConf, *m_conf, "arbitration.solo-auto");
+        copySetting<int>(tmpConf, *m_conf, "arbitration.solo-players");
         copySetting<int>(tmpConf, *m_conf, "arbitration.solo-value");
         copySetting<int>(tmpConf, *m_conf, "arbitration.penalty-value");
         copySetting<int>(tmpConf, *m_conf, "arbitration.warnings-limit");
+        copySetting<bool>(tmpConf, *m_conf, "topping.elapsed-penalty");
+        copySetting<int>(tmpConf, *m_conf, "topping.timeout-penalty");
     }
-    catch (...)
+    catch (const std::exception &e)
     {
-        // Ignore the exception
+        // Only log the exception
+        LOG_ERROR("Error reading config file: " << e.what());
     }
 #endif
 }
@@ -347,12 +369,16 @@ int Settings::getInt(const string &iName) const
         return 10;
     else if (iName == "arbitration.search-limit")
         return 100;
+    else if (iName == "arbitration.solo-players")
+        return 16;
     else if (iName == "arbitration.solo-value")
         return 5;
     else if (iName == "arbitration.penalty-value")
         return 5;
     else if (iName == "arbitration.warnings-limit")
         return 3;
+    else if (iName == "topping.timeout-penalty")
+        return 60;
     return 0;
 #endif
 }
