@@ -19,9 +19,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-#include <algorithm>
-#include <cwctype> // For towupper
-
 #include "config.h"
 #if ENABLE_NLS
 #   include <libintl.h>
@@ -39,10 +36,10 @@
 #include "move.h"
 #include "pldrack.h"
 #include "player.h"
-#include "player_move_cmd.h"
-#include "player_rack_cmd.h"
-#include "game_move_cmd.h"
-#include "game_rack_cmd.h"
+#include "cmd/player_move_cmd.h"
+#include "cmd/player_rack_cmd.h"
+#include "cmd/game_move_cmd.h"
+#include "cmd/game_rack_cmd.h"
 #include "encoding.h"
 
 #include "debug.h"
@@ -51,8 +48,8 @@
 INIT_LOGGER(game, Training);
 
 
-Training::Training(const GameParams &iParams)
-    : Game(iParams), m_results(1000)
+Training::Training(const GameParams &iParams, const Game *iMasterGame)
+    : Game(iParams, iMasterGame), m_results(1000)
 {
 }
 
@@ -73,13 +70,14 @@ void Training::setRackRandom(bool iCheck, set_rack_mode mode)
 
 void Training::setRackManual(bool iCheck, const wstring &iLetters)
 {
+    ASSERT(!hasMasterGame(),
+           "Changing the rack is not allowed when there is a master game");
+
     // Letters can be lowercase or uppercase as they are
     // coming from user input. We do not consider a lowercase
     // letter to be a joker which has been assigned to a letter.
     // As a result, we simply make all the letters uppercase
-    wstring upperLetters = iLetters;
-    std::transform(upperLetters.begin(), upperLetters.end(),
-                   upperLetters.begin(), towupper);
+    wstring upperLetters = toUpper(iLetters);
     const PlayedRack &newRack = helperSetRackManual(iCheck, upperLetters);
     Command *pCmd1 = new GameRackCmd(*this, newRack);
     pCmd1->setHumanIndependent(false);
@@ -94,18 +92,14 @@ void Training::setRackManual(bool iCheck, const wstring &iLetters)
 
 int Training::play(const wstring &iCoord, const wstring &iWord)
 {
-    // Perform all the validity checks, and fill a round
-    Round round;
-
     m_board.removeTestRound();
 
-    int res = checkPlayedWord(iCoord, iWord, round);
+    // Perform all the validity checks, and fill a move
+    Move move;
+    int res = checkPlayedWord(iCoord, iWord, move);
     if (res != 0)
-    {
         return res;
-    }
 
-    Move move(round);
     recordPlayerMove(move, *m_players[m_currPlayer]);
 
     // Next turn
@@ -146,8 +140,12 @@ void Training::endTurn()
     m_results.clear();
 
     // Play the word on the board
-    const Move &move = m_players[m_currPlayer]->getLastMove();
-    Command *pCmd = new GameMoveCmd(*this, move, m_currPlayer);
+    Move move;
+    if (hasMasterGame())
+        move = getMoveFromMasterGame();
+    else
+        move = m_players[m_currPlayer]->getLastMove();
+    Command *pCmd = new GameMoveCmd(*this, move);
     accessNavigation().addAndExecute(pCmd);
     accessNavigation().newTurn();
 }
