@@ -53,12 +53,12 @@
 #include "player.h"
 #include "ai_percent.h"
 #include "encoding.h"
-#include "game_rack_cmd.h"
-#include "game_move_cmd.h"
-#include "player_rack_cmd.h"
-#include "player_move_cmd.h"
-#include "player_event_cmd.h"
-#include "master_move_cmd.h"
+#include "cmd/game_rack_cmd.h"
+#include "cmd/game_move_cmd.h"
+#include "cmd/player_rack_cmd.h"
+#include "cmd/player_move_cmd.h"
+#include "cmd/player_event_cmd.h"
+#include "cmd/master_move_cmd.h"
 #include "navigation.h"
 #include "header.h"
 
@@ -117,10 +117,11 @@ static int toInt(const string &str)
 }
 
 
-static Player & getPlayer(map<string, Player*> &players, const string &id)
+static Player & getPlayer(map<string, Player*> &players,
+                          const string &id, const string &iTag)
 {
     if (players.find(id) == players.end())
-        throw LoadGameException(FMT1(_("Invalid player ID: %1%"), id));
+        throw LoadGameException(FMT2(_("Invalid player ID: %1% (processing tag '%2%')"), id, iTag));
     return *players[id];
 }
 
@@ -133,15 +134,15 @@ static Move buildMove(const Game &iGame, map<string, string> &attr,
     if (type == "valid")
     {
         wstring word = iGame.getDic().convertFromInput(fromUtf8(attr["word"]));
-        Round round;
+        Move move;
         int res = iGame.checkPlayedWord(fromUtf8(attr["coord"]),
-                                        word, round, checkRack);
+                                        word, move, checkRack);
         if (res != 0)
         {
             throw LoadGameException(FMT2(_("Invalid move marked as valid: %1% (%2%)"),
                                          attr["word"], attr["coord"]));
         }
-        return Move(round);
+        return move;
     }
     else if (type == "invalid")
     {
@@ -226,6 +227,10 @@ void XmlReader::startElement(const string& namespaceURI,
         else
             m_game->accessNavigation().newTurn();
     }
+    // For backwards compatibility ("playerid" was used in saved games
+    // before release 2.1, with XML version 2)
+    if (m_attributes["playerid"] != "")
+        m_attributes["playerId"] = m_attributes["playerid"];
 }
 
 
@@ -277,6 +282,8 @@ void XmlReader::endElement(const string& namespaceURI,
             m_params.setMode(GameParams::kTRAINING);
         else if (m_data == "arbitration")
             m_params.setMode(GameParams::kARBITRATION);
+        else if (m_data == "topping")
+            m_params.setMode(GameParams::kTOPPING);
         else
             throw GameException("Invalid game mode: " + m_data);
         return;
@@ -313,6 +320,8 @@ void XmlReader::endElement(const string& namespaceURI,
             m_attributes["type"] = m_data;
         else if (tag == "Level")
             m_attributes["level"] = m_data;
+        else if (tag == "TableNb")
+            m_attributes["tablenb"] = m_data;
         else if (tag == "Player")
         {
             if (m_players.find(m_attributes["id"]) != m_players.end())
@@ -332,6 +341,9 @@ void XmlReader::endElement(const string& namespaceURI,
 
             // Set the name
             p->setName(fromUtf8(m_attributes["name"]));
+            // Ste the table number
+            if (m_attributes["tablenb"] != "")
+                p->setTableNb(toInt(m_attributes["tablenb"]));
 
             m_game->addPlayer(p);
 
@@ -368,7 +380,7 @@ void XmlReader::endElement(const string& namespaceURI,
         pldrack.setManual(rackStr);
         LOG_DEBUG("loaded rack: " << lfw(pldrack.toString()));
 
-        Player &p = getPlayer(m_players, m_attributes["playerid"]);
+        Player &p = getPlayer(m_players, m_attributes["playerId"], tag);
         PlayerRackCmd *cmd = new PlayerRackCmd(p, pldrack);
         m_game->accessNavigation().addAndExecute(cmd);
         LOG_DEBUG("rack: " << lfw(pldrack.toString()));
@@ -388,30 +400,32 @@ void XmlReader::endElement(const string& namespaceURI,
 
     else if (tag == "PlayerMove")
     {
+        // FIXME: this is game-related logic. It should not be done here.
+        bool isArbitrationGame = m_game->getParams().getMode() == GameParams::kARBITRATION;
+
         const Move &move = buildMove(*m_game, m_attributes, /*XXX:true*/false);
-        Player &p = getPlayer(m_players, m_attributes["playerid"]);
-        PlayerMoveCmd *cmd = new PlayerMoveCmd(p, move);
+        Player &p = getPlayer(m_players, m_attributes["playerId"], tag);
+        PlayerMoveCmd *cmd = new PlayerMoveCmd(p, move, isArbitrationGame);
         m_game->accessNavigation().addAndExecute(cmd);
     }
 
     else if (tag == "GameMove")
     {
         const Move &move = buildMove(*m_game, m_attributes, false);
-        Player &p = getPlayer(m_players, m_attributes["playerid"]);
-        GameMoveCmd *cmd = new GameMoveCmd(*m_game, move, p.getId());
+        GameMoveCmd *cmd = new GameMoveCmd(*m_game, move);
         m_game->accessNavigation().addAndExecute(cmd);
     }
 
     else if (tag == "Warning")
     {
-        Player &p = getPlayer(m_players, m_attributes["playerid"]);
+        Player &p = getPlayer(m_players, m_attributes["playerId"], tag);
         PlayerEventCmd *cmd = new PlayerEventCmd(p, PlayerEventCmd::WARNING);
         m_game->accessNavigation().addAndExecute(cmd);
     }
 
     else if (tag == "Penalty")
     {
-        Player &p = getPlayer(m_players, m_attributes["playerid"]);
+        Player &p = getPlayer(m_players, m_attributes["playerId"], tag);
         int points = toInt(m_attributes["points"]);
         LOG_ERROR("points=" << points);
         PlayerEventCmd *cmd = new PlayerEventCmd(p, PlayerEventCmd::PENALTY, points);
@@ -420,7 +434,7 @@ void XmlReader::endElement(const string& namespaceURI,
 
     else if (tag == "Solo")
     {
-        Player &p = getPlayer(m_players, m_attributes["playerid"]);
+        Player &p = getPlayer(m_players, m_attributes["playerId"], tag);
         int points = toInt(m_attributes["points"]);
         PlayerEventCmd *cmd = new PlayerEventCmd(p, PlayerEventCmd::SOLO, points);
         m_game->accessNavigation().addAndExecute(cmd);
@@ -428,7 +442,7 @@ void XmlReader::endElement(const string& namespaceURI,
 
     else if (tag == "EndGame")
     {
-        Player &p = getPlayer(m_players, m_attributes["playerid"]);
+        Player &p = getPlayer(m_players, m_attributes["playerId"], tag);
         int points = toInt(m_attributes["points"]);
         PlayerEventCmd *cmd = new PlayerEventCmd(p, PlayerEventCmd::END_GAME, points);
         m_game->accessNavigation().addAndExecute(cmd);
