@@ -1,6 +1,7 @@
 /*****************************************************************************
- * Copyright (C) 2009 Eliot
- * Authors: Olivier Teuliere  <ipkiss@via.ecp.fr>
+ * Eliot
+ * Copyright (C) 2009-2012 Olivier Teulière
+ * Authors: Olivier Teulière  <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +22,38 @@
 #define XML_READER_H_
 
 #include <map>
+
+// Remove spurious defines, to avoid compilation warnings. They are defined
+// in config.h (which is normal) and in libarabica (which is wrong:
+// libarabica should not export them via public headers).
+// We do it a first time before including the libarabica headers (in case
+// config.h was included before the current header)...
+#undef PACKAGE
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
+#undef VERSION
+
 #include <SAX/helpers/DefaultHandler.hpp>
 #include <SAX/Locator.hpp>
 #include <SAX/Attributes.hpp>
 #include <SAX/SAXException.hpp>
+
+// ... and a second time after including them (because we include
+// config.h right after)
+#undef PACKAGE
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
+#undef VERSION
+// We include config.h, because a cpp file which included it expects
+// these defines
+#include "config.h"
+
 #include "logging.h"
 #include "game_params.h"
 
@@ -73,6 +102,7 @@ private:
     const Dictionary &m_dic;
     Game *m_game;
     string errorMessage;
+    bool m_firstTurn;
 
     string m_context;
     string m_data;
@@ -83,7 +113,7 @@ private:
     // Private constructor, because we only want the read() method
     // to be called externally
     XmlReader(const Dictionary &iDic) :
-        m_dic(iDic), m_game(NULL), m_params(iDic) {}
+        m_dic(iDic), m_game(NULL), m_firstTurn(true), m_params(iDic) {}
 
     XmlReader(const XmlReader&);
     XmlReader& operator=(const XmlReader&);
