@@ -35,18 +35,20 @@
 #include "xml_writer.h"
 #include "encoding.h"
 #include "turn.h"
+#include "turn_data.h"
 #include "game_params.h"
 #include "game.h"
 #include "player.h"
 #include "ai_percent.h"
 #include "game_exception.h"
 #include "turn.h"
-#include "game_rack_cmd.h"
-#include "game_move_cmd.h"
-#include "player_rack_cmd.h"
-#include "player_move_cmd.h"
-#include "player_event_cmd.h"
-#include "master_move_cmd.h"
+#include "cmd/game_rack_cmd.h"
+#include "cmd/game_move_cmd.h"
+#include "cmd/player_rack_cmd.h"
+#include "cmd/player_move_cmd.h"
+#include "cmd/player_event_cmd.h"
+#include "cmd/master_move_cmd.h"
+#include "cmd/topping_move_cmd.h"
 #include "dic.h"
 #include "header.h"
 
@@ -84,7 +86,7 @@ static void writeMove(ostream &out, const Move &iMove,
 {
     out << "<" << iTag;
     if (iPlayerId != -1)
-        out << " playerid=\"" << iPlayerId << "\"";
+        out << " playerId=\"" << iPlayerId << "\"";
     out << " points=\"" << iMove.getScore() << "\" type=\"";
     if (iMove.isValid())
     {
@@ -162,6 +164,8 @@ void XmlWriter::write(const Game &iGame, const string &iFileName)
         out << "freegame";
     else if (iGame.getMode() == GameParams::kARBITRATION)
         out << "arbitration";
+    else if (iGame.getMode() == GameParams::kTOPPING)
+        out << "topping";
     else
         out << "training";
     out << "</Mode>" << endl;
@@ -178,7 +182,7 @@ void XmlWriter::write(const Game &iGame, const string &iFileName)
     for (unsigned int i = 0; i < iGame.getNPlayers(); ++i)
     {
         const Player &player = iGame.getPlayer(i);
-        out << indent << "<Player id=\"" << player.getId() + 1 << "\">" << endl;
+        out << indent << "<Player id=\"" << player.getId() << "\">" << endl;
         addIndent(indent);
         out << indent << "<Name>" << toUtf8(player.getName()) << "</Name>" << endl;
         out << indent << "<Type>" << (player.isHuman() ? "human" : "computer") << "</Type>" << endl;
@@ -189,6 +193,7 @@ void XmlWriter::write(const Game &iGame, const string &iFileName)
                 throw SaveGameException(FMT1(_("Invalid player type for player %1%"), i));
             out << indent << "<Level>" << lrint(ai->getPercent() * 100) << "</Level>" << endl;
         }
+        out << indent << "<TableNb>" << player.getTableNb() << "</TableNb>" << endl;
         removeIndent(indent);
         out << indent << "</Player>" << endl;
     }
@@ -215,6 +220,7 @@ void XmlWriter::write(const Game &iGame, const string &iFileName)
     {
         if (turn->getCommands().empty() && turn == turnVect.back())
             continue;
+
         out << indent << "<Turn>" << endl;
         addIndent(indent);
         BOOST_FOREACH(const Command *cmd, turn->getCommands())
@@ -229,15 +235,15 @@ void XmlWriter::write(const Game &iGame, const string &iFileName)
             else if (dynamic_cast<const PlayerRackCmd*>(cmd))
             {
                 const PlayerRackCmd *rackCmd = static_cast<const PlayerRackCmd*>(cmd);
-                unsigned int id = rackCmd->getPlayer().getId() + 1;
-                out << indent << "<PlayerRack playerid=\"" << id << "\">"
+                unsigned int id = rackCmd->getPlayer().getId();
+                out << indent << "<PlayerRack playerId=\"" << id << "\">"
                     << toUtf8(rackCmd->getRack().toString())
                     << "</PlayerRack>" << endl;
             }
             else if (dynamic_cast<const PlayerMoveCmd*>(cmd))
             {
                 const PlayerMoveCmd *moveCmd = static_cast<const PlayerMoveCmd*>(cmd);
-                unsigned int id = moveCmd->getPlayer().getId() + 1;
+                unsigned int id = moveCmd->getPlayer().getId();
                 out << indent;
                 writeMove(out, moveCmd->getMove(), "PlayerMove", id);
                 out << endl;
@@ -245,9 +251,8 @@ void XmlWriter::write(const Game &iGame, const string &iFileName)
             else if (dynamic_cast<const GameMoveCmd*>(cmd))
             {
                 const GameMoveCmd *moveCmd = static_cast<const GameMoveCmd*>(cmd);
-                unsigned int id = moveCmd->getPlayerId() + 1;
                 out << indent;
-                writeMove(out, moveCmd->getMove(), "GameMove", id);
+                writeMove(out, moveCmd->getMove(), "GameMove", -1);
                 out << endl;
             }
             else if (dynamic_cast<const MasterMoveCmd*>(cmd))
@@ -256,34 +261,42 @@ void XmlWriter::write(const Game &iGame, const string &iFileName)
                 out << indent;
                 writeMove(out, moveCmd->getMove(), "MasterMove", -1);
                 out << endl;
-
+            }
+            else if (dynamic_cast<const ToppingMoveCmd*>(cmd))
+            {
+                const ToppingMoveCmd *moveCmd = static_cast<const ToppingMoveCmd*>(cmd);
+                unsigned int id = moveCmd->getPlayerId();
+                out << indent;
+                // FIXME: the elapsed time is not saved
+                writeMove(out, moveCmd->getMove(), "ToppingMove", id);
+                out << endl;
             }
             else if (dynamic_cast<const PlayerEventCmd*>(cmd))
             {
                 const PlayerEventCmd *eventCmd = static_cast<const PlayerEventCmd*>(cmd);
-                unsigned int id = eventCmd->getPlayer().getId() + 1;
+                unsigned int id = eventCmd->getPlayer().getId();
                 int value = eventCmd->getPoints();
                 // Warnings
                 if (eventCmd->getEventType() == PlayerEventCmd::WARNING)
                 {
-                    out << indent << "<Warning playerid=\"" << id << "\" />" << endl;
+                    out << indent << "<Warning playerId=\"" << id << "\" />" << endl;
                 }
                 // Penalties
                 else if (eventCmd->getEventType() == PlayerEventCmd::PENALTY)
                 {
-                    out << indent << "<Penalty playerid=\"" << id
+                    out << indent << "<Penalty playerId=\"" << id
                         << "\" points=\"" << value << "\" />" << endl;
                 }
                 // Solos
                 else if (eventCmd->getEventType() == PlayerEventCmd::SOLO)
                 {
-                    out << indent << "<Solo playerid=\"" << id
+                    out << indent << "<Solo playerId=\"" << id
                         << "\" points=\"" << value << "\" />" << endl;
                 }
                 // End game bonuses (freegame mode)
                 else if (eventCmd->getEventType() == PlayerEventCmd::END_GAME)
                 {
-                    out << indent << "<EndGame playerid=\"" << id
+                    out << indent << "<EndGame playerId=\"" << id
                         << "\" points=\"" << value << "\" />" << endl;
                 }
                 else
@@ -308,6 +321,62 @@ void XmlWriter::write(const Game &iGame, const string &iFileName)
     // End of the game history
     // ------------------------
 
+    // Statistics
+    out << indent << "<!-- These statistics are simply informative, "
+        << "they are not used when loading a game. -->" << endl;
+    out << indent << "<Statistics>" << endl;
+    addIndent(indent);
+
+    // Compute the total number of points in the game
+    int gameTotal = 0;
+    for (unsigned i = 0; i < iGame.getHistory().getSize(); ++i)
+    {
+        gameTotal += iGame.getHistory().getTurn(i).getMove().getScore();
+    }
+
+    out << indent << "<GameStats"
+        << " totalScore=\"" << gameTotal << "\""
+        << " />" << endl;
+
+    for (unsigned int i = 0; i < iGame.getNPlayers(); ++i)
+    {
+        const Player &player = iGame.getPlayer(i);
+        const int playerTotal = player.getTotalScore();
+
+        // Compute the percentage, compared to the top
+        long int percentage = 0;
+        if (gameTotal != 0)
+            percentage = lround((double)100 * playerTotal / gameTotal);
+
+        // Compute the rank of the player (naive algorithm)
+        int rank = 1;
+        for (unsigned j = 0; j < iGame.getNPlayers(); ++j)
+        {
+            if (i == j)
+                continue;
+            if (playerTotal < iGame.getPlayer(j).getTotalScore())
+                ++rank;
+        }
+
+        out << indent << "<PlayerStats"
+            << " playerId=\"" << player.getId() << "\"" << endl;
+        addIndent(indent);
+        out << indent << " rawScore=\"" << player.getMovePoints() << "\""
+            << " warningsNb=\"" << player.getWarningsNb() << "\""
+            << " penaltiesPoints=\"" << player.getPenaltyPoints() << "\""
+            << " solosPoints=\"" << player.getSoloPoints() << "\"" << endl;
+        out << indent << " totalScore=\"" << playerTotal << "\""
+            << " diffWithTop=\"" << (playerTotal - gameTotal) << "\""
+            << " percentTop=\"" << percentage << "%\""
+            << " rank=\"" << rank << "\""
+            << " />" << endl;
+        removeIndent(indent);
+    }
+
+    removeIndent(indent);
+    out << indent << "</Statistics>" << endl;
+    // End of the statistics
+
     out << "</EliotGame>" << endl;
 }
 
