@@ -23,17 +23,19 @@
 #include <QtGui/QStandardItemModel>
 #include <QtGui/QSortFilterProxyModel>
 #include <QtGui/QMenu>
+#include <QtGui/QShortcut>
 
 #include "arbit_assignments.h"
 #include "qtcommon.h"
 #include "custom_popup.h"
-#include "misc_helpers.h"
 #include "prefs_dialog.h"
 
 #include "public_game.h"
 #include "player.h"
+#include "turn_data.h"
 #include "rack.h"
 #include "results.h"
+#include "settings.h"
 #include "debug.h"
 
 using namespace std;
@@ -54,49 +56,63 @@ ArbitAssignments::ArbitAssignments(QWidget *parent, PublicGame *iGame)
     m_playersModel = new QStandardItemModel(this);
     m_proxyPlayersModel->setSourceModel(m_playersModel);
     treeViewPlayers->setModel(m_proxyPlayersModel);
-    m_playersModel->setColumnCount(5);
+    m_playersModel->setColumnCount(8);
     m_playersModel->setHeaderData(0, Qt::Horizontal, _q("Table"), Qt::DisplayRole);
     m_playersModel->setHeaderData(1, Qt::Horizontal, _q("Player"), Qt::DisplayRole);
     m_playersModel->setHeaderData(2, Qt::Horizontal, _q("Word"), Qt::DisplayRole);
     m_playersModel->setHeaderData(3, Qt::Horizontal, _q("Ref"), Qt::DisplayRole);
     m_playersModel->setHeaderData(4, Qt::Horizontal, _q("Points"), Qt::DisplayRole);
+    m_playersModel->setHeaderData(5, Qt::Horizontal, _q("S"), Qt::DisplayRole);
+    m_playersModel->setHeaderData(6, Qt::Horizontal, _q("W"), Qt::DisplayRole);
+    m_playersModel->setHeaderData(7, Qt::Horizontal, _q("P"), Qt::DisplayRole);
     treeViewPlayers->sortByColumn(0, Qt::AscendingOrder);
 
     treeViewPlayers->setColumnWidth(0, 70);
-    treeViewPlayers->setColumnWidth(1, 160);
-    treeViewPlayers->setColumnWidth(2, 160);
+    treeViewPlayers->setColumnWidth(1, 150);
+    treeViewPlayers->setColumnWidth(2, 130);
     treeViewPlayers->setColumnWidth(3, 40);
     treeViewPlayers->setColumnWidth(4, 50);
+    treeViewPlayers->setColumnWidth(5, 25);
+    treeViewPlayers->setColumnWidth(6, 25);
+    treeViewPlayers->setColumnWidth(7, 25);
 
-    KeyEventFilter *filter = new KeyEventFilter(this, Qt::Key_A);
-    QObject::connect(filter, SIGNAL(keyPressed(int, int)),
-                     this, SLOT(selectAllPlayers()));
-    treeViewPlayers->installEventFilter(filter);
-
-    filter = new KeyEventFilter(this, Qt::Key_Delete);
-    QObject::connect(filter, SIGNAL(keyPressed(int, int)),
+    QShortcut *shortcut;
+    shortcut = new QShortcut(QKeySequence::Delete, treeViewPlayers);
+    shortcut->setContext(Qt::WidgetWithChildrenShortcut);
+    QObject::connect(shortcut, SIGNAL(activated()),
                      this, SLOT(suppressMove()));
-    treeViewPlayers->installEventFilter(filter);
 
-    filter = new KeyEventFilter(this, Qt::Key_T);
-    QObject::connect(filter, SIGNAL(keyPressed(int, int)),
+    // TRANSLATORS: 'T' is the keyboard shortcut used in arbitration mode
+    // to assign the top move to players. If translated, the translation
+    // will be used as shortcut instead of 'T'.
+    shortcut = new QShortcut(_q("T"), treeViewPlayers);
+    shortcut->setContext(Qt::WidgetWithChildrenShortcut);
+    QObject::connect(shortcut, SIGNAL(activated()),
                      this, SLOT(assignTopMove()));
-    treeViewPlayers->installEventFilter(filter);
 
-    filter = new KeyEventFilter(this, Qt::Key_S);
-    QObject::connect(filter, SIGNAL(keyPressed(int, int)),
+    // TRANSLATORS: 'S' stands for Solo, it is used as column header in
+    // the history, and in the players table in arbitration mode. It is also
+    // used as shortcut to assign a solo, in arbitration mode.
+    shortcut = new QShortcut(_q("S"), treeViewPlayers);
+    shortcut->setContext(Qt::WidgetWithChildrenShortcut);
+    QObject::connect(shortcut, SIGNAL(activated()),
                      this, SLOT(addRemoveSolo()));
-    treeViewPlayers->installEventFilter(filter);
 
-    filter = new KeyEventFilter(this, Qt::Key_W);
-    QObject::connect(filter, SIGNAL(keyPressed(int, int)),
+    // TRANSLATORS: 'W' stands for Warning, it is used as column header in
+    // the history, and in the players table in arbitration mode. It is also
+    // used as shortcut to assign a warning, in arbitration mode.
+    shortcut = new QShortcut(_q("W"), treeViewPlayers);
+    shortcut->setContext(Qt::WidgetWithChildrenShortcut);
+    QObject::connect(shortcut, SIGNAL(activated()),
                      this, SLOT(addRemoveWarning()));
-    treeViewPlayers->installEventFilter(filter);
 
-    filter = new KeyEventFilter(this, Qt::Key_P);
-    QObject::connect(filter, SIGNAL(keyPressed(int, int)),
+    // TRANSLATORS: 'P' stands for Penalty, it is used as column header in
+    // the history, and in the players table in arbitration mode. It is also
+    // used as shortcut to assign a penalty, in arbitration mode.
+    shortcut = new QShortcut(_q("P"), treeViewPlayers);
+    shortcut->setContext(Qt::WidgetWithChildrenShortcut);
+    QObject::connect(shortcut, SIGNAL(activated()),
                      this, SLOT(addRemovePenalty()));
-    treeViewPlayers->installEventFilter(filter);
 
     // Display a preview of the master word when clicked
     QObject::connect(labelMasterMove, SIGNAL(clicked()),
@@ -114,7 +130,7 @@ ArbitAssignments::ArbitAssignments(QWidget *parent, PublicGame *iGame)
 
     // Move assignment
     QObject::connect(buttonSelectMaster, SIGNAL(clicked()),
-                     this, SLOT(assignMasterMove()));
+                     this, SLOT(setMasterMove()));
     QObject::connect(buttonSuppressMove, SIGNAL(clicked()),
                      this, SLOT(suppressMove()));
     QObject::connect(buttonAssign, SIGNAL(clicked()),
@@ -158,7 +174,7 @@ void ArbitAssignments::refresh()
     setEnabled(!m_game->isFinished());
 
     enableAssignmentButtons();
-    buttonSelectMaster->setEnabled(isAssignMasterAllowed());
+    buttonSelectMaster->setEnabled(isSetMasterAllowed());
     buttonEndTurn->setEnabled(isEndTurnAllowed());
 }
 
@@ -224,6 +240,16 @@ void ArbitAssignments::updatePlayersModel()
             m_playersModel->setData(m_playersModel->index(rowNum, 3),
                                     brush, Qt::ForegroundRole);
         }
+
+        // Print the solos, warnings and penalties
+        const TurnData &turnData = player.getHistory().getTurn(m_game->getCurrTurn() - 1);
+        if (turnData.getSoloPoints() != 0)
+            m_playersModel->setData(m_playersModel->index(rowNum, 5), turnData.getSoloPoints());
+        if (turnData.getWarningsNb() != 0)
+            m_playersModel->setData(m_playersModel->index(rowNum, 6), turnData.getWarningsNb());
+        if (turnData.getPenaltyPoints() != 0)
+            m_playersModel->setData(m_playersModel->index(rowNum, 7), turnData.getPenaltyPoints());
+
         // Restore the selection
         if (playersIdSet.contains(player.getId()))
         {
@@ -255,7 +281,7 @@ void ArbitAssignments::enableAssignmentButtons()
                                  .arg(formatMove(move)));
     }
     buttonSuppressMove->setEnabled(isSuppressMoveAllowed());
-    buttonSelectMaster->setEnabled(isAssignMasterAllowed());
+    buttonSelectMaster->setEnabled(isSetMasterAllowed());
 }
 
 
@@ -266,23 +292,27 @@ void ArbitAssignments::populatePlayersMenu(QMenu &iMenu, const QPoint &iPoint)
         return;
 
     // Action to assign the selected move
+    QString selMoveString = _q("none");
     if (isAssignMoveAllowed())
     {
         const Move &move = m_selectedMove;
-        QAction *assignSelMoveAction =
-            new QAction(_q("Assign selected move (%1)").arg(formatMove(move)), this);
-        assignSelMoveAction->setStatusTip(_q("Assign move (%1) to the selected player(s)")
-                        .arg(formatMove(move)));
-        assignSelMoveAction->setShortcut(Qt::Key_Enter);
-        QObject::connect(assignSelMoveAction, SIGNAL(triggered()),
-                         this, SLOT(assignSelectedMove()));
-        iMenu.addAction(assignSelMoveAction);
+        selMoveString = formatMove(move);
     }
+    QAction *assignSelMoveAction =
+        new QAction(_q("Assign selected move (%1)").arg(selMoveString), this);
+    assignSelMoveAction->setStatusTip(_q("Assign move (%1) to the selected player(s)")
+                    .arg(selMoveString));
+    assignSelMoveAction->setShortcut(Qt::Key_Enter);
+    QObject::connect(assignSelMoveAction, SIGNAL(triggered()),
+                     this, SLOT(assignSelectedMove()));
+    iMenu.addAction(assignSelMoveAction);
+    if (!isAssignMoveAllowed())
+        assignSelMoveAction->setEnabled(false);
 
     // Action to assign the top move
     QAction *assignTopMoveAction = new QAction(_q("Assign top move (if unique)"), this);
     assignTopMoveAction->setStatusTip(_q("Assign the top move (if unique) to the selected player(s)"));
-    assignTopMoveAction->setShortcut(Qt::Key_T);
+    assignTopMoveAction->setShortcut(_q("T"));
     QObject::connect(assignTopMoveAction, SIGNAL(triggered()),
                      this, SLOT(assignTopMove()));
     iMenu.addAction(assignTopMoveAction);
@@ -300,7 +330,7 @@ void ArbitAssignments::populatePlayersMenu(QMenu &iMenu, const QPoint &iPoint)
     // Action to select all the players
     QAction *selectAllAction = new QAction(_q("Select all players"), this);
     selectAllAction->setStatusTip(_q("Select all the players"));
-    selectAllAction->setShortcut(Qt::Key_A);
+    selectAllAction->setShortcut(QKeySequence::SelectAll);
     QObject::connect(selectAllAction, SIGNAL(triggered()),
                      this, SLOT(selectAllPlayers()));
     iMenu.addAction(selectAllAction);
@@ -308,15 +338,17 @@ void ArbitAssignments::populatePlayersMenu(QMenu &iMenu, const QPoint &iPoint)
     // Action to give or remove a solo to players
     QAction *soloAction = new QAction(_q("Give (or remove) a solo"), this);
     soloAction->setStatusTip(_q("Give a solo to the selected player, or remove it if (s)he already has one"));
-    soloAction->setShortcut(Qt::Key_S);
+    soloAction->setShortcut(_q("S"));
     QObject::connect(soloAction, SIGNAL(triggered()),
                      this, SLOT(addRemoveSolo()));
     iMenu.addAction(soloAction);
+    if (useSoloAuto())
+        soloAction->setEnabled(false);
 
     // Action to give or remove a warning to players
     QAction *warningAction = new QAction(_q("Give (or remove) a warning"), this);
     warningAction->setStatusTip(_q("Give a warning to the selected player(s), or remove it if they already have one"));
-    warningAction->setShortcut(Qt::Key_W);
+    warningAction->setShortcut(_q("W"));
     QObject::connect(warningAction, SIGNAL(triggered()),
                      this, SLOT(addRemoveWarning()));
     iMenu.addAction(warningAction);
@@ -324,7 +356,7 @@ void ArbitAssignments::populatePlayersMenu(QMenu &iMenu, const QPoint &iPoint)
     // Action to give or remove a penalty to players
     QAction *penaltyAction = new QAction(_q("Give (or remove) a penalty"), this);
     penaltyAction->setStatusTip(_q("Give a penalty to the selected player(s), or remove it if they already have one"));
-    penaltyAction->setShortcut(Qt::Key_P);
+    penaltyAction->setShortcut(_q("P"));
     QObject::connect(penaltyAction, SIGNAL(triggered()),
                      this, SLOT(addRemovePenalty()));
     iMenu.addAction(penaltyAction);
@@ -394,7 +426,7 @@ void ArbitAssignments::showMasterPreview()
     const Move &move = m_game->duplicateGetMasterMove();
     if (move.isValid())
     {
-        // TODO: deselect move in the Results?
+        // TODO: deselect move in the Results? or reselect the master move?
         m_game->setTestRound(move.getRound());
         emit gameUpdated();
     }
@@ -408,15 +440,16 @@ void ArbitAssignments::selectAllPlayers()
 }
 
 
-bool ArbitAssignments::isAssignMasterAllowed() const
+bool ArbitAssignments::isSetMasterAllowed() const
 {
-    return m_game->isLastTurn() && m_selectedMove.isValid();
+    return m_game->isLastTurn() && m_selectedMove.isValid() &&
+        !m_game->hasMasterGame();
 }
 
 
-void ArbitAssignments::assignMasterMove()
+void ArbitAssignments::setMasterMove()
 {
-    if (!isAssignMasterAllowed())
+    if (!isSetMasterAllowed())
         return;
 
     const Move &masterMove = m_game->duplicateGetMasterMove();
@@ -439,7 +472,7 @@ void ArbitAssignments::assignMasterMove()
     results.search(m_game->getDic(), m_game->getBoard(),
                    m_game->getCurrentRack().getRack(),
                    m_game->getHistory().beforeFirstRound());
-    ASSERT(results.size() != 0, "No possible valid move");
+    ASSERT(!results.isEmpty(), "No possible valid move");
     int bestScore = results.get(0).getPoints();
     if (bestScore > move.getScore())
     {
@@ -484,7 +517,7 @@ void ArbitAssignments::assignMasterMove()
 }
 
 
-void ArbitAssignments::assignDefaultMasterMove()
+void ArbitAssignments::setDefaultMasterMove()
 {
     const Move &currMove = m_game->duplicateGetMasterMove();
     // Do not overwrite an existing move
@@ -492,30 +525,18 @@ void ArbitAssignments::assignDefaultMasterMove()
         return;
 
     // Search the best moves
-    BestResults results;
+    MasterResults results(m_game->getBag());
     results.search(m_game->getDic(), m_game->getBoard(),
                    m_game->getCurrentRack().getRack(),
                    m_game->getHistory().beforeFirstRound());
     // XXX: End of game
-    if (results.size() == 0)
+    if (results.isEmpty())
         return;
 
-    unsigned currIndex = 0;
-    unsigned jokerCount = results.get(0).countJokersFromRack();
-    if (jokerCount > 0)
-    {
-        for (unsigned i = 1; i < results.size(); ++i)
-        {
-            if (results.get(i).countJokersFromRack() < jokerCount)
-            {
-                currIndex = i;
-                jokerCount = results.get(i).countJokersFromRack();
-            }
-        }
-    }
+    // Find a good default
+    Move move = Move(results.get(0));
 
     // Assign the master move
-    Move move = Move(results.get(currIndex));
     m_game->duplicateSetMasterMove(move);
     emit gameUpdated();
 }
@@ -637,8 +658,17 @@ void ArbitAssignments::helperAssignMove(const Move &iMove)
 }
 
 
+bool ArbitAssignments::useSoloAuto() const
+{
+    return Settings::Instance().getBool("arbitration.solo-auto");
+}
+
+
 void ArbitAssignments::addRemoveSolo()
 {
+    if (useSoloAuto())
+        return;
+
     QSet<unsigned int> playersIdSet = getSelectedPlayers();
     // Only one player can have a solo
     if (playersIdSet.size() != 1)
