@@ -21,7 +21,9 @@
 #include <QtGui/QStandardItemModel>
 #include <QtGui/QSortFilterProxyModel>
 #include <QtGui/QMenu>
+#include <QtGui/QShortcut>
 #include <QtCore/QSettings>
+#include <QtCore/QSignalMapper>
 
 #include "arbitration_widget.h"
 #include "arbit_assignments.h"
@@ -30,13 +32,12 @@
 #include "validator_factory.h"
 #include "play_word_mediator.h"
 #include "custom_popup.h"
-#include "misc_helpers.h"
+#include "play_model.h"
 
 #include "public_game.h"
 #include "player.h"
 #include "rack.h"
 #include "results.h"
-#include "coord_model.h"
 #include "settings.h"
 #include "game_params.h"
 #include "dic.h"
@@ -54,8 +55,8 @@ INIT_LOGGER(qt, ArbitrationWidget);
 
 
 ArbitrationWidget::ArbitrationWidget(QWidget *parent,
-                                     PublicGame *iGame, CoordModel &iCoordModel)
-    : QWidget(parent), m_game(iGame), m_coordModel(iCoordModel), m_results(10)
+                                     PublicGame *iGame, PlayModel &iPlayModel)
+    : QWidget(parent), m_game(iGame), m_playModel(iPlayModel), m_results(10)
 {
     setupUi(this);
 
@@ -115,30 +116,30 @@ ArbitrationWidget::ArbitrationWidget(QWidget *parent,
     treeViewResults->setColumnWidth(1, 40);
     treeViewResults->setColumnWidth(2, 70);
 
-    KeyEventFilter *masterFilter = new KeyEventFilter(this, Qt::Key_M, Qt::SHIFT);
-    QObject::connect(masterFilter, SIGNAL(keyPressed(int, int)),
-                     m_assignmentsWidget, SLOT(assignMasterMove()));
-    treeViewResults->installEventFilter(masterFilter);
+    QShortcut *shortcut;
+    shortcut = new QShortcut(QString("Shift+M"), treeViewResults);
+    shortcut->setContext(Qt::WidgetWithChildrenShortcut);
+    QObject::connect(shortcut, SIGNAL(activated()),
+                     m_assignmentsWidget, SLOT(setMasterMove()));
 
-    KeyEventFilter *selectAllFilter = new KeyEventFilter(this, Qt::Key_A, Qt::SHIFT);
-    QObject::connect(selectAllFilter, SIGNAL(keyPressed(int, int)),
+    shortcut = new QShortcut(QString("Shift+A"), treeViewResults);
+    shortcut->setContext(Qt::WidgetWithChildrenShortcut);
+    QObject::connect(shortcut, SIGNAL(activated()),
                      m_assignmentsWidget, SLOT(selectAllPlayers()));
-    treeViewResults->installEventFilter(selectAllFilter);
 
-    KeyEventFilter *numFilter = new KeyEventFilter(this, Qt::Key_0);
-    numFilter->addKey(Qt::Key_1);
-    numFilter->addKey(Qt::Key_2);
-    numFilter->addKey(Qt::Key_3);
-    numFilter->addKey(Qt::Key_4);
-    numFilter->addKey(Qt::Key_5);
-    numFilter->addKey(Qt::Key_6);
-    numFilter->addKey(Qt::Key_7);
-    numFilter->addKey(Qt::Key_8);
-    numFilter->addKey(Qt::Key_9);
-    numFilter->setIgnoreModifiers();
-    QObject::connect(numFilter, SIGNAL(keyPressed(int, int)),
-                     this, SLOT(selectTableNumber(int)));
-    treeViewResults->installEventFilter(numFilter);
+    // React to digits (0 to 9)
+    m_signalMapper = new QSignalMapper(this);
+    for (int i = 0; i <= 9; ++i)
+    {
+        QString numStr = QString("%1").arg(i);
+        shortcut = new QShortcut(numStr, treeViewResults);
+        shortcut->setContext(Qt::WidgetWithChildrenShortcut);
+        QObject::connect(shortcut, SIGNAL(activated()),
+                         m_signalMapper, SLOT(map()));
+        m_signalMapper->setMapping(shortcut, numStr);
+    }
+    QObject::connect(m_signalMapper, SIGNAL(mapped(const QString&)),
+                     this, SLOT(selectTableNumber(const QString&)));
 
     // Validate manual rack changes
     QObject::connect(lineEditRack, SIGNAL(textEdited(const QString&)),
@@ -166,9 +167,11 @@ ArbitrationWidget::ArbitrationWidget(QWidget *parent,
                      SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)),
                      this, SLOT(showPreview(const QItemSelection&)));
 
-    // Dynamic filter for search results
-    QObject::connect(lineEditFilter, SIGNAL(textChanged(const QString&)),
-                     this, SLOT(resultsFilterChanged(const QString&)));
+    // Dynamic filters for search results
+    QObject::connect(lineEditFilterWord, SIGNAL(textChanged(const QString&)),
+                     this, SLOT(resultsFilterWordChanged(const QString&)));
+    QObject::connect(lineEditFilterPoints, SIGNAL(textChanged(const QString&)),
+                     this, SLOT(resultsFilterPointsChanged(const QString&)));
 
     // Enable the assignment buttons according to the selections in trees
     QObject::connect(treeViewResults->selectionModel(),
@@ -181,10 +184,10 @@ ArbitrationWidget::ArbitrationWidget(QWidget *parent,
                      this, SLOT(updateSelectedMove()));
 
     // Coordinates model
-    QObject::connect(&m_coordModel, SIGNAL(coordChanged(const Coord&, const Coord&)),
-                     this, SLOT(updateCoordText(const Coord&, const Coord&)));
+    QObject::connect(&m_playModel, SIGNAL(coordChanged(const Coord&, const Coord&)),
+                     this, SLOT(updateCoordText(const Coord&)));
     QObject::connect(lineEditCoords, SIGNAL(textChanged(const QString&)),
-                     this, SLOT(updateCoordModel(const QString&)));
+                     this, SLOT(updatePlayModel(const QString&)));
 
     // Enable the "Check word" button only when there is a word with coordinates
     QObject::connect(lineEditWord, SIGNAL(textChanged(const QString&)),
@@ -240,10 +243,11 @@ void ArbitrationWidget::refresh()
     updateResultsModel();
     m_assignmentsWidget->refresh();
 
-    // The rack can only be changed at the last turn
+    // The rack can only be changed at the last turn,
+    // and if there is no master game
     bool isLastTurn = m_game->isLastTurn();
-    lineEditRack->setReadOnly(!isLastTurn);
-    buttonRandom->setEnabled(isLastTurn);
+    lineEditRack->setReadOnly(!isLastTurn || m_game->hasMasterGame());
+    buttonRandom->setEnabled(isLastTurn && !m_game->hasMasterGame());
 
     // Disable controls when the game is finished, but only
     // for the last turn (to allow changing players moves in
@@ -288,11 +292,6 @@ void ArbitrationWidget::updateResultsModel()
         LOG_DEBUG("Adding custom move: " << lfw(move.toString()));
         addSingleMove(move, TYPE_ADDED, i, bestScore);
     }
-
-    // FIXME arbitration begin
-    // XXX: useful?
-    //m_proxyResultsModel->invalidate();
-    // FIXME arbitration end
 }
 
 
@@ -437,18 +436,46 @@ void ArbitrationWidget::rackChanged()
     buttonSearch->setEnabled(acceptableInput);
     lineEditRack->setPalette(acceptableInput ? blackPalette : redPalette);
     clearResults();
+
+    // Clear the filters
+    lineEditFilterWord->blockSignals(true);
+    lineEditFilterWord->clear();
+    lineEditFilterWord->blockSignals(false);
+    lineEditFilterPoints->blockSignals(true);
+    lineEditFilterPoints->clear();
+    lineEditFilterPoints->blockSignals(false);
 }
 
 
-void ArbitrationWidget::resultsFilterChanged(const QString &iFilter)
+void ArbitrationWidget::resultsFilterWordChanged(const QString &iFilter)
 {
+    // Clear the points filter
+    lineEditFilterPoints->blockSignals(true);
+    lineEditFilterPoints->clear();
+    lineEditFilterPoints->blockSignals(false);
+
     treeViewResults->clearSelection();
+    m_proxyResultsModel->setFilterKeyColumn(0);
     m_proxyResultsModel->setFilterFixedString(iFilter);
     emit gameUpdated();
 }
 
 
-void ArbitrationWidget::updateCoordText(const Coord &, const Coord &iNewCoord)
+void ArbitrationWidget::resultsFilterPointsChanged(const QString &iFilter)
+{
+    // Clear the word filter
+    lineEditFilterWord->blockSignals(true);
+    lineEditFilterWord->clear();
+    lineEditFilterWord->blockSignals(false);
+
+    treeViewResults->clearSelection();
+    m_proxyResultsModel->setFilterKeyColumn(2);
+    m_proxyResultsModel->setFilterFixedString(iFilter);
+    emit gameUpdated();
+}
+
+
+void ArbitrationWidget::updateCoordText(const Coord &iNewCoord)
 {
     if (iNewCoord.isValid() && lineEditCoords->text() != qfw(iNewCoord.toString()))
         lineEditCoords->setText(qfw(iNewCoord.toString()));
@@ -458,11 +485,10 @@ void ArbitrationWidget::updateCoordText(const Coord &, const Coord &iNewCoord)
 }
 
 
-void ArbitrationWidget::updateCoordModel(const QString &iText)
+void ArbitrationWidget::updatePlayModel(const QString &iText)
 {
     Coord coord(wfq(iText));
-    if (!(m_coordModel.getCoord() == coord))
-        m_coordModel.setCoord(coord);
+    m_playModel.setCoord(coord);
 }
 
 
@@ -477,7 +503,6 @@ void ArbitrationWidget::searchResults()
 {
     m_game->removeTestRound();
     emit notifyInfo(_q("Searching with rack '%1'...").arg(lineEditRack->text()));
-    lineEditFilter->clear();
     m_results.clear();
     m_game->arbitrationSearch(m_results);
     emit notifyInfo(_q("Search done"));
@@ -486,7 +511,7 @@ void ArbitrationWidget::searchResults()
     QSettings settings;
     if (settings.value(PrefsDialog::kARBIT_AUTO_MASTER, false).toBool())
     {
-        m_assignmentsWidget->assignDefaultMasterMove();
+        m_assignmentsWidget->setDefaultMasterMove();
     }
 
     // Set the focus to the first result
@@ -517,9 +542,9 @@ void ArbitrationWidget::populateResultsMenu(QMenu &iMenu, const QPoint &iPoint)
                                     .arg(formatMove(move)));
     setAsMasterAction->setShortcut(Qt::SHIFT + Qt::Key_M);
     QObject::connect(setAsMasterAction, SIGNAL(triggered()),
-                     m_assignmentsWidget, SLOT(assignMasterMove()));
+                     m_assignmentsWidget, SLOT(setMasterMove()));
     iMenu.addAction(setAsMasterAction);
-    if (!m_assignmentsWidget->isAssignMasterAllowed())
+    if (!m_assignmentsWidget->isSetMasterAllowed())
         setAsMasterAction->setEnabled(false);
 
     // Action to select all the players
@@ -596,23 +621,10 @@ void ArbitrationWidget::selectAndFocusResult(int iRowNum, bool logical)
 }
 
 
-void ArbitrationWidget::selectTableNumber(int key)
+void ArbitrationWidget::selectTableNumber(const QString &iKey)
 {
-    QString keyStr = "";
-    if (key == Qt::Key_0) keyStr = "0";
-    else if (key == Qt::Key_1) keyStr = "1";
-    else if (key == Qt::Key_2) keyStr = "2";
-    else if (key == Qt::Key_3) keyStr = "3";
-    else if (key == Qt::Key_4) keyStr = "4";
-    else if (key == Qt::Key_5) keyStr = "5";
-    else if (key == Qt::Key_6) keyStr = "6";
-    else if (key == Qt::Key_7) keyStr = "7";
-    else if (key == Qt::Key_8) keyStr = "8";
-    else if (key == Qt::Key_9) keyStr = "9";
-    ASSERT(keyStr != "", "Unexpected key");
-
     // Build (and retrieve) the table number
-    QString tableNum = m_keyAccum->addText(keyStr);
+    QString tableNum = m_keyAccum->addText(iKey);
 
     // Select the player with this table number
     LOG_DEBUG("Selecting player with table number: " + lfq(tableNum));
@@ -710,7 +722,7 @@ int ArbitrationWidget::getBestScore() const
     results.search(m_game->getDic(), m_game->getBoard(),
                    m_game->getCurrentRack().getRack(),
                    m_game->getHistory().beforeFirstRound());
-    ASSERT(results.size() != 0, "No possible valid move");
+    ASSERT(!results.isEmpty(), "No possible valid move");
     return results.get(0).getPoints();
 }
 
