@@ -25,6 +25,7 @@
 #include <QtGui/QWidget>
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QCloseEvent>
+#include <QtGui/QDesktopWidget>
 #include <QtCore/QSettings>
 
 #include "aux_window.h"
@@ -107,7 +108,16 @@ void AuxWindow::readSettings()
     QSize size = settings.value("size").toSize();
     if (size.isValid())
         resize(size);
-    move(settings.value("pos", QPoint(200, 200)).toPoint());
+    const QRect &desktopRect = QApplication::desktop()->screenGeometry();
+    QPoint point = settings.value("pos", QPoint(20, 20)).toPoint();
+    // If the position was saved when an external monitor was plugged, and
+    // is restored when the monitor is not there anymore, the window could
+    // be off screen...
+    if (point.x() < 0 || point.x() > desktopRect.right())
+        point.setX(20);
+    if (point.y() < 0 || point.y() > desktopRect.bottom())
+        point.setY(20);
+    move(point);
     settings.endGroup();
 }
 
