@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,15 +21,23 @@
 #ifndef BAG_WIDGET_H_
 #define BAG_WIDGET_H_
 
-#include <QtGui/QTreeView>
+#include <vector>
+#include <QtGui/QWidget>
 
+#include <ui/bag_widget.ui.h>
+#include "logging.h"
+
+using std::vector;
 
-class PublicGame;
 class QStandardItemModel;
+class PublicGame;
+class TileWidget;
 
-class BagWidget: public QTreeView
+
+class BagWidget: public QWidget, private Ui::BagWidget
 {
     Q_OBJECT;
+    DEFINE_LOGGER();
 
 public:
     explicit BagWidget(QWidget *parent = 0);
@@ -46,6 +54,8 @@ private:
     /// Encapsulated game, can be NULL
     const PublicGame *m_game;
 
+//QTreeView *m_treeView;
+
     /// Model of the bag
     QStandardItemModel *m_model;
 
@@ -53,5 +63,42 @@ private:
     void updateModel();
 };
 
+
+class BagWidget2: public QWidget
+{
+    Q_OBJECT;
+    DEFINE_LOGGER();
+
+public:
+    explicit BagWidget2(QWidget *parent = 0);
+
+public slots:
+    void setGame(const PublicGame *iGame);
+    void setShowPlayedTiles(bool iShow);
+    void setShowTilesInRack(bool iShow);
+    void refresh();
+
+protected:
+    /// Define a default size
+    virtual QSize sizeHint() const;
+
+private:
+    /// Encapsulated game, can be NULL
+    const PublicGame *m_game;
+
+    /// Encapsulated tiles
+    vector<TileWidget *> m_tilesVect;
+
+    /// Cached value, storing the total number of tiles in the game
+    unsigned int m_totalNbTiles;
+
+    /// Indicate whether the played tiles should be displayed
+    bool m_showPlayedTiles;
+
+    /// Indicate whether the tiles in the current rack should be displayed highlighted
+    bool m_showTilesInRack;
+
+};
+
 #endif
 
