@@ -28,8 +28,10 @@
 #include "qtcommon.h"
 #include "public_game.h"
 #include "tile.h"
+#include "board_layout.h"
 #include "board.h"
-#include "coord_model.h"
+#include "play_model.h"
+#include "move.h"
 
 using namespace std;
 
@@ -37,12 +39,17 @@ using namespace std;
 INIT_LOGGER(qt, BoardWidget);
 
 
-BoardWidget::BoardWidget(CoordModel &iCoordModel, QWidget *parent)
+BoardWidget::BoardWidget(PlayModel &iPlayModel, QWidget *parent)
     : QFrame(parent), m_game(NULL),
-    m_coordModel(iCoordModel), m_showTemporarySigns(true),
-    m_showOnlyLastTurn(false),
-    m_widgetsMatrix(BOARD_MAX + 1, BOARD_MAX + 1, 0)
+    m_playModel(iPlayModel), m_showTemporarySigns(true),
+    m_showOnlyLastTurn(false)
 {
+    const BoardLayout & boardLayout = BoardLayout::GetDefault();
+    const unsigned nbRows = boardLayout.getRowCount();
+    const unsigned nbCols = boardLayout.getColCount();
+
+    m_widgetsMatrix.resize(nbRows + 1, nbCols + 1, 0);
+
     // Try to have a black background... FIXME: not working well!
     QPalette pal = palette();
     for (int i = 0; i <= 19; ++i)
@@ -51,36 +58,39 @@ BoardWidget::BoardWidget(CoordModel &iCoordModel, QWidget *parent)
     setForegroundRole(QPalette::Window);
     setBackgroundRole(QPalette::Window);
 
-    TileLayout *layout = new TileLayout(BOARD_MAX + 1, BOARD_MAX + 1);
+    TileLayout *layout = new TileLayout(nbRows + 1, nbCols + 1);
     layout->setSpacing(1);
+    layout->setAlignment(Qt::AlignHCenter);
+
     // Line full of coordinates
     TileWidget *cornerTile = new TileWidget;
     cornerTile->setCoordText("");
     layout->addWidget(cornerTile);
-    for (unsigned int col = BOARD_MIN; col <= BOARD_MAX; ++col)
+    for (unsigned int col = 1; col <= nbCols; ++col)
     {
         TileWidget *coordTile = new TileWidget;
         coordTile->setCoordText(QString("%1").arg(col));
         layout->addWidget(coordTile);
     }
+
     // Rest of the board
-    for (unsigned int row = BOARD_MIN; row <= BOARD_MAX; ++row)
+    for (unsigned int row = 1; row <= nbRows; ++row)
     {
         // Add the coordinate
         TileWidget *coordTile = new TileWidget;
-        coordTile->setCoordText(QString(QChar('A' + row - BOARD_MIN)));
+        coordTile->setCoordText(QString(QChar('A' + row - 1)));
         layout->addWidget(coordTile);
         // Add the squares
-        for (unsigned int col = BOARD_MIN; col <= BOARD_MAX; ++col)
+        for (unsigned int col = 1; col <= nbCols; ++col)
         {
             TileWidget::Multiplier mult = TileWidget::NONE;
-            if (Board::GetWordMultiplier(row, col) == 3)
+            if (boardLayout.getWordMultiplier(row, col) == 3)
                 mult = TileWidget::WORD_TRIPLE;
-            else if (Board::GetWordMultiplier(row, col) == 2)
+            else if (boardLayout.getWordMultiplier(row, col) == 2)
                 mult = TileWidget::WORD_DOUBLE;
-            else if (Board::GetLetterMultiplier(row, col) == 3)
+            else if (boardLayout.getLetterMultiplier(row, col) == 3)
                 mult = TileWidget::LETTER_TRIPLE;
-            else if (Board::GetLetterMultiplier(row, col) == 2)
+            else if (boardLayout.getLetterMultiplier(row, col) == 2)
                 mult = TileWidget::LETTER_DOUBLE;
             TileWidget *t = new TileWidget(this, mult, row, col);
             m_widgetsMatrix[row][col] = t;
@@ -98,8 +108,10 @@ BoardWidget::BoardWidget(CoordModel &iCoordModel, QWidget *parent)
     setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
 
     // Listen to changes in the coordinates
-    QObject::connect(&m_coordModel, SIGNAL(coordChanged(const Coord&, const Coord&)),
+    QObject::connect(&m_playModel, SIGNAL(coordChanged(const Coord&, const Coord&)),
                      this, SLOT(updateArrow(const Coord&, const Coord&)));
+    QObject::connect(&m_playModel, SIGNAL(moveChanged(const Move&, const Move&)),
+                     this, SLOT(onMoveChanged(const Move&)));
 }
 
 
@@ -110,7 +122,7 @@ void BoardWidget::setGame(const PublicGame *iGame)
 }
 
 
-void BoardWidget::updateArrow(const Coord &iOldCoord, const Coord &iNewCoord)
+void BoardWidget::updateArrow(const Coord &iNewCoord, const Coord &iOldCoord)
 {
     // Refresh only the 2 involved squares
     if (iOldCoord.isValid())
@@ -130,6 +142,22 @@ void BoardWidget::updateArrow(const Coord &iOldCoord, const Coord &iNewCoord)
 }
 
 
+void BoardWidget::onMoveChanged(const Move &iMove)
+{
+    if (m_game == NULL)
+        return;
+
+    // FIXME
+    Board &board = const_cast<Board&>(m_game->getBoard());
+    board.removeTestRound();
+    if (iMove.isValid())
+    {
+        board.testRound(iMove.getRound());
+    }
+    refresh();
+}
+
+
 void BoardWidget::refresh()
 {
     if (m_game != NULL)
@@ -139,9 +167,11 @@ void BoardWidget::refresh()
         // Note: the TileWidget class will redraw the tile only if something
         // has changed, to avoid useless repainting.
         const Board &board = m_game->getBoard();
-        for (unsigned int row = BOARD_MIN; row <= BOARD_MAX; ++row)
+        const unsigned nbRows = board.getLayout().getRowCount();
+        const unsigned nbCols = board.getLayout().getColCount();
+        for (unsigned row = 1; row <= nbRows; ++row)
         {
-            for (unsigned int col = BOARD_MIN; col <= BOARD_MAX; ++col)
+            for (unsigned col = 1; col <= nbCols; ++col)
             {
                 if (board.isTestChar(row, col) && m_showTemporarySigns)
                 {
@@ -171,9 +201,9 @@ void BoardWidget::refresh()
     else
     {
         // Clear the board
-        for (unsigned int row = BOARD_MIN; row <= BOARD_MAX; ++row)
+        for (unsigned row = 1; row < m_widgetsMatrix.size(); ++row)
         {
-            for (unsigned int col = BOARD_MIN; col <= BOARD_MAX; ++col)
+            for (unsigned col = 1; col < m_widgetsMatrix[1].size(); ++col)
             {
                 m_widgetsMatrix[row][col]->tileChanged(TileWidget::BOARD_EMPTY);
             }
@@ -197,18 +227,19 @@ void BoardWidget::paintEvent(QPaintEvent *)
     const int size = boardLayout->getSquareSize();
     const int spacing = boardLayout->spacing();
 
+    // Draw lines between tiles
     QLine hLine(0, 0, rect.width() + 1, 0);
     QLine vLine(0, 0, 0, rect.height() + 1);
-    hLine.translate(size, size);
-    vLine.translate(size, size);
-    for (int i = 0; i <= BOARD_MAX; ++i)
+    hLine.translate(rect.left() - 1, rect.top() - 1);
+    vLine.translate(rect.left() - 1, rect.top() - 1);
+    for (unsigned i = 0; i < m_widgetsMatrix.size(); ++i)
     {
         painter.drawLine(hLine);
         painter.drawLine(vLine);
         hLine.translate(0, size + spacing);
         vLine.translate(size + spacing, 0);
     }
-    //painter.drawRect(rect);
+    // Draw a second line around the board, so that it looks nicer
     painter.drawRect(rect.adjusted(-2, -2, 1, 1));
 }
 
@@ -217,7 +248,7 @@ void BoardWidget::tileClicked(int row, int col, QMouseEvent *iEvent)
 {
     if (m_game == NULL)
     {
-        m_coordModel.clear();
+        m_playModel.clear();
         return;
     }
 
@@ -229,15 +260,15 @@ void BoardWidget::tileClicked(int row, int col, QMouseEvent *iEvent)
     {
         // Change the direction if this is exactly the same as the current one
         Coord coord(row, col, Coord::HORIZONTAL);
-        if (m_coordModel.getCoord() == coord)
+        if (m_playModel.getCoord() == coord)
             coord.setDir(Coord::VERTICAL);
         // Take into acount the new coordinates
-        m_coordModel.setCoord(coord);
+        m_playModel.setCoord(coord);
     }
     else if (iEvent->button() == Qt::RightButton)
     {
         // On a right click anywhere on the board, remove the arrow
-        m_coordModel.clear();
+        m_playModel.clear();
     }
 #endif
 #if 1
@@ -248,24 +279,24 @@ void BoardWidget::tileClicked(int row, int col, QMouseEvent *iEvent)
     {
         // Change the direction if this is exactly the same as the current one
         Coord coord(row, col, Coord::HORIZONTAL);
-        if (m_coordModel.getCoord().getRow() == coord.getRow() &&
-            m_coordModel.getCoord().getCol() == coord.getCol())
+        if (m_playModel.getCoord().getRow() == coord.getRow() &&
+            m_playModel.getCoord().getCol() == coord.getCol())
         {
-            if (m_coordModel.getCoord().getDir() == Coord::VERTICAL)
+            if (m_playModel.getCoord().getDir() == Coord::VERTICAL)
             {
                 // Third click: clear the arrow
-                m_coordModel.clear();
+                m_playModel.clear();
                 return;
             }
             coord.setDir(Coord::VERTICAL);
         }
         // Take into acount the new coordinates
-        m_coordModel.setCoord(coord);
+        m_playModel.setCoord(coord);
     }
     else if (iEvent->button() == Qt::RightButton)
     {
         // On a right click anywhere on the board, remove the arrow
-        m_coordModel.clear();
+        m_playModel.clear();
     }
 #endif
 #if 0
@@ -277,23 +308,23 @@ void BoardWidget::tileClicked(int row, int col, QMouseEvent *iEvent)
         Coord coord(row, col, Coord::HORIZONTAL);
         // Remove the coordinates if they are exactly the same as the current ones,
         // otherwise set the coordinates;
-        if (m_coordModel.getCoord() == coord)
-            m_coordModel.clear();
+        if (m_playModel.getCoord() == coord)
+            m_playModel.clear();
         else
-            m_coordModel.setCoord(coord);
+            m_playModel.setCoord(coord);
     }
     else if (iEvent->button() == Qt::RightButton)
     {
         Coord coord(row, col, Coord::VERTICAL);
         // Remove the coordinates if they are exactly the same as the current ones,
         // otherwise set the coordinates;
-        if (m_coordModel.getCoord() == coord)
-            m_coordModel.clear();
+        if (m_playModel.getCoord() == coord)
+            m_playModel.clear();
         else
-            m_coordModel.setCoord(coord);
+            m_playModel.setCoord(coord);
     }
     else
-        m_coordModel.clear();
+        m_playModel.clear();
 #endif
 }
 
