@@ -29,8 +29,9 @@
 
 class PublicGame;
 class TileWidget;
-class CoordModel;
+class PlayModel;
 class Coord;
+class Move;
 
 class BoardWidget: public QFrame
 {
@@ -38,7 +39,7 @@ class BoardWidget: public QFrame
     DEFINE_LOGGER();
 
 public:
-    explicit BoardWidget(CoordModel &iCoordModel, QWidget *parent = 0);
+    explicit BoardWidget(PlayModel &iPlayModel, QWidget *parent = 0);
     void setShowTempSigns(bool iShow) { m_showTemporarySigns = iShow; }
     void setShowOnlyLastTurn(bool iShow) { m_showOnlyLastTurn = iShow; }
 
@@ -56,14 +57,15 @@ protected:
 
 private slots:
     void tileClicked(int row, int col, QMouseEvent *iEvent);
-    void updateArrow(const Coord &iOldCoord, const Coord &iNewCoord);
+    void updateArrow(const Coord &iNewCoord, const Coord &iOldCoord);
+    void onMoveChanged(const Move &iMove);
 
 private:
     /// Encapsulated game, can be NULL
     const PublicGame *m_game;
 
     /// Coordinates of the next word to play
-    CoordModel &m_coordModel;
+    PlayModel &m_playModel;
 
     /// Indicate whether to draw the arrow and the previewed word on the board
     bool m_showTemporarySigns;
