@@ -18,11 +18,12 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *****************************************************************************/
 
-#ifndef VALIDATORS_H_
-#define VALIDATORS_H_
+#ifndef PLAY_WORD_MEDIATOR_H_
+#define PLAY_WORD_MEDIATOR_H_
 
 #include <string>
 #include <QtGui/QValidator>
+#include <QtGui/QPalette>
 
 #include "logging.h"
 
@@ -33,7 +34,7 @@ class QLineEdit;
 class QPushButton;
 class PublicGame;
 class Coord;
-class CoordModel;
+class PlayModel;
 class Dictionary;
 
 /**
@@ -56,9 +57,9 @@ class PlayWordMediator: public QObject
 
 public:
     PlayWordMediator(QObject *parent, QLineEdit &iEditWord,
-                     QLineEdit &iEditCoord, QLineEdit &iEditPoints,
+                     QLineEdit &iEditCoord, QLineEdit *iEditPoints,
                      QPushButton &iButtonPlay,
-                     CoordModel &iCoordModel, PublicGame *iGame);
+                     PlayModel &iPlayModel, PublicGame *iGame);
 
     /**
      * Set the focus on the editWord QLineEdit, unless the focus
@@ -90,16 +91,24 @@ signals:
 private slots:
     void playWord();
     void updatePointsAndState();
-    void onCoordChanged(const QString &iText);
-    void updateCoord(const Coord &, const Coord &iNewCoord);
+    void onCoordChanged();
+    void onWordChanged();
+    void updateCoord(const Coord &iNewCoord);
 
 private:
     PublicGame *m_game;
     QLineEdit &m_lineEditPlay;
     QLineEdit &m_lineEditCoord;
-    QLineEdit &m_lineEditPoints;
+    QLineEdit *m_lineEditPoints;
     QPushButton &m_pushButtonPlay;
-    CoordModel &m_coordModel;
+    PlayModel &m_playModel;
+
+    /// Palette to write text in black
+    QPalette blackPalette;
+
+    /// Palette to write text in red
+    QPalette redPalette;
+
 
     /**
      * Wrapper around GetPlayedWord(), more practical to use.
