@@ -29,7 +29,7 @@
 #include "player.h"
 #include "pldrack.h"
 #include "coord.h"
-#include "coord_model.h"
+#include "play_model.h"
 #include "dic.h"
 #include "debug.h"
 
@@ -39,7 +39,7 @@
 INIT_LOGGER(qt, PlayerWidget);
 
 
-PlayerWidget::PlayerWidget(QWidget *parent, CoordModel &iCoordModel,
+PlayerWidget::PlayerWidget(QWidget *parent, PlayModel &iPlayModel,
                            unsigned int iPlayerNb, PublicGame *iGame)
     : QWidget(parent), m_game(iGame), m_player(iPlayerNb)
 {
@@ -47,8 +47,8 @@ PlayerWidget::PlayerWidget(QWidget *parent, CoordModel &iCoordModel,
 
     // Use the mediator
     m_mediator = new PlayWordMediator(this, *lineEditPlay, *lineEditCoords,
-                                      *lineEditPoints, *pushButtonPlay,
-                                      iCoordModel, iGame);
+                                      lineEditPoints, *pushButtonPlay,
+                                      iPlayModel, iGame);
     QObject::connect(m_mediator, SIGNAL(gameUpdated()),
                      this, SIGNAL(gameUpdated()));
     QObject::connect(m_mediator, SIGNAL(notifyProblem(QString)),
@@ -115,7 +115,7 @@ void PlayerWidget::refresh()
     lineEditCoords->clear();
     lineEditChange->clear();
 
-    if (!m_game->isLastTurn())
+    if (!m_game->isLastTurn() || m_game->isFinished())
     {
         // Do not allow entering a move when displaying an old turn
         setEnabled(false);
@@ -201,8 +201,8 @@ void PlayerWidget::helperChangePass(QString inputLetters)
 
 
 
-PlayerTabWidget::PlayerTabWidget(CoordModel &iCoordModel, QWidget *parent)
-    : QTabWidget(parent), m_game(NULL), m_coordModel(iCoordModel)
+PlayerTabWidget::PlayerTabWidget(PlayModel &iPlayModel, QWidget *parent)
+    : QTabWidget(parent), m_game(NULL), m_playModel(iPlayModel)
 {
     QObject::connect(this, SIGNAL(currentChanged(int)),
                      this, SLOT(changeCurrentPlayer(int)));
@@ -215,6 +215,7 @@ void PlayerTabWidget::setGame(PublicGame *iGame)
 
     // Remove all the tabs
     int nbTabs = count();
+    blockSignals(true);
     for (int i = 0; i < nbTabs; ++i)
     {
         setCurrentIndex(0);
@@ -223,14 +224,16 @@ void PlayerTabWidget::setGame(PublicGame *iGame)
         disconnect(currentWidget());
         removeTab(0);
     }
+    blockSignals(false);
 
     if (iGame != NULL)
     {
+        blockSignals(true);
         // Add one tab per player
         for (unsigned int i = 0; i < iGame->getNbPlayers(); ++i)
         {
             const Player &player = iGame->getPlayer(i);
-            PlayerWidget *p = new PlayerWidget(NULL, m_coordModel, i, iGame);
+            PlayerWidget *p = new PlayerWidget(NULL, m_playModel, i, iGame);
             QObject::connect(this, SIGNAL(refreshSignal()), p, SLOT(refresh()));
             // Forward signals to the outside
             QObject::connect(p, SIGNAL(notifyProblem(QString)),
@@ -244,6 +247,7 @@ void PlayerTabWidget::setGame(PublicGame *iGame)
                 setTabEnabled(i, false);
         }
         setCurrentIndex(iGame->getCurrentPlayer().getId());
+        blockSignals(false);
     }
 }
 
@@ -262,12 +266,14 @@ void PlayerTabWidget::refresh()
 
 void PlayerTabWidget::changeCurrentPlayer(int p)
 {
+#if 0
     // This method is triggered somehow when creating a Duplicate game
     // after a FreeGame one. The next line avoids crashing in this case...
     if (m_game == NULL)
         return;
+#endif
 
-    m_coordModel.clear();
+    m_playModel.clear();
 
     // Change the active player when the active tab changes
     // (only in duplicate mode)
@@ -275,6 +281,7 @@ void PlayerTabWidget::changeCurrentPlayer(int p)
         widget(p)->isEnabled())
     {
         m_game->duplicateSetPlayer(p);
+        emit gameUpdated();
     }
 }
 
