@@ -43,6 +43,8 @@ const QString PrefsDialog::kINTF_SHOW_TOOLBAR = "Interface/ShowToolBar";
 const QString PrefsDialog::kINTF_TIMER_TOTAL_DURATION = "Interface/TimerTotalDuration";
 const QString PrefsDialog::kINTF_TIMER_ALERT_DURATION = "Interface/TimerAlertDuration";
 const QString PrefsDialog::kINTF_TIMER_BEEPS = "Interface/TimerBeeps";
+const QString PrefsDialog::kINTF_TIMER_AUTO_START = "Interface/TimerAutoStart";
+const QString PrefsDialog::kINTF_CHECK_FOR_UPDATES = "Interface/CheckForUpdates";
 const QString PrefsDialog::kARBIT_AUTO_MASTER = "Arbitration/AutoAssignMaster";
 const QString PrefsDialog::kARBIT_LINK_7P1 = "Arbitration/LinkRackWith7P1";
 const QString PrefsDialog::kDEFAULT_DEF_SITE = "http://fr.wiktionary.org/wiki/%w";
@@ -84,6 +86,11 @@ PrefsDialog::PrefsDialog(QWidget *iParent)
                                      "Changing this value will reset the timer."));
     checkBoxTimerBeeps->setToolTip(_q("If checked, a beep will be emitted when the timer\n"
                                       "reaches the alert level, and when it reaches 0."));
+    checkBoxTimerAutoStart->setToolTip(_q("If checked, the timer will be reinitialized and restarted\n"
+                                          "automatically every time that the main rack changes."));
+    checkBoxIntfCheckUpdates->setToolTip(_q("If checked, Eliot will connect to the Internet from time to time\n"
+                                            "(about once a week) to check if new versions are available.\n"
+                                            "New versions are never installed automatically, you just get a notification."));
     spinBoxTrainSearchLimit->setToolTip(_q("Maximum number of results returned by a search.\n"
                                            "The returned results will always be the best ones.\n"
                                            "Use 0 to disable the limit (warning: searches yielding many "
@@ -93,9 +100,15 @@ PrefsDialog::PrefsDialog(QWidget *iParent)
                                            "It is still possible to change the master move afterwards."));
     checkBoxArbitFillRack->setToolTip(_q("If checked, the rack will be completed with random letters.\n"
                                          "Uncheck this option if you prefer to choose the letters yourself."));
+    checkBoxArbitSoloAuto->setToolTip(_q("If checked, solos are given automatically, when appropriate.\n"
+                                         "Uncheck this option if you prefer to do it manually."));
     spinBoxArbitSearchLimit->setToolTip(spinBoxTrainSearchLimit->toolTip());
     spinBoxArbitWarnLimit->setToolTip(_q("Maximal number of \"acceptable\" warnings.\n"
                                          "Any additional warning will give a penalty to the player."));
+    checkBoxToppingElapsedPenalty->setToolTip(_q("If checked, the player gets, at each turn, a score penalty\n"
+                                                 "equal to the elapsed time for the turn."));
+    spinBoxToppingExpPenalty->setToolTip(_q("Number of points added to the player score when the timer expires.\n"
+                                            "Set it to 0 if you don't want any penalty."));
 
     // Auto-completion on the dictionary path
     QCompleter *completer = new QCompleter(this);
@@ -104,6 +117,13 @@ PrefsDialog::PrefsDialog(QWidget *iParent)
     completer->setModel(model);
     lineEditIntfDicPath->setCompleter(completer);
 
+    // The "arbitration.solo-players" setting is meaningful only
+    // when "arbitration.solo-auto" is true
+    QObject::connect(checkBoxArbitSoloAuto, SIGNAL(toggled(bool)),
+                     labelArbitSoloPlayers, SLOT(setEnabled(bool)));
+    QObject::connect(checkBoxArbitSoloAuto, SIGNAL(toggled(bool)),
+                     spinBoxArbitSoloPlayers, SLOT(setEnabled(bool)));
+
     try
     {
         // Interface settings
@@ -120,6 +140,10 @@ PrefsDialog::PrefsDialog(QWidget *iParent)
         spinBoxTimerAlert->setValue(timerAlert);
         bool timerBeeps = qs.value(kINTF_TIMER_BEEPS, true).toBool();
         checkBoxTimerBeeps->setChecked(timerBeeps);
+        bool timerAutoStart = qs.value(kINTF_TIMER_AUTO_START, false).toBool();
+        checkBoxTimerAutoStart->setChecked(timerAutoStart);
+        bool checkForUpdates = qs.value(kINTF_CHECK_FOR_UPDATES, true).toBool();
+        checkBoxIntfCheckUpdates->setChecked(checkForUpdates);
 
         // Duplicate settings
         checkBoxDuplRefuseInvalid->setChecked(Settings::Instance().getBool("duplicate.reject-invalid"));
@@ -138,10 +162,16 @@ PrefsDialog::PrefsDialog(QWidget *iParent)
         checkBoxArbitFillRack->setChecked(Settings::Instance().getBool("arbitration.fill-rack"));
         bool linkArbit7P1 = qs.value(kARBIT_LINK_7P1, false).toBool();
         checkBoxArbitLink7P1->setChecked(linkArbit7P1);
-        spinBoxArbitSearchLimit->setValue(Settings::Instance().getInt("arbitration.search-limit"));
+        checkBoxArbitSoloAuto->setChecked(Settings::Instance().getBool("arbitration.solo-auto"));
+        spinBoxArbitSoloPlayers->setValue(Settings::Instance().getInt("arbitration.solo-players"));
+        spinBoxArbitSoloValue->setValue(Settings::Instance().getInt("arbitration.solo-value"));
         spinBoxArbitPenaltyValue->setValue(Settings::Instance().getInt("arbitration.penalty-value"));
         spinBoxArbitWarnLimit->setValue(Settings::Instance().getInt("arbitration.warnings-limit"));
-        spinBoxArbitSoloValue->setValue(Settings::Instance().getInt("arbitration.solo-value"));
+        spinBoxArbitSearchLimit->setValue(Settings::Instance().getInt("arbitration.search-limit"));
+
+        // Topping settings
+        checkBoxToppingElapsedPenalty->setChecked(Settings::Instance().getBool("topping.elapsed-penalty"));
+        spinBoxToppingExpPenalty->setValue(Settings::Instance().getInt("topping.timeout-penalty"));
 
         // Confirmations
         bool confoStartGame = qs.value(kCONFO_START_GAME, true).toBool();
@@ -228,6 +258,13 @@ void PrefsDialog::updateSettings()
             qs.setValue(kINTF_TIMER_ALERT_DURATION, spinBoxTimerAlert->value());
         }
         qs.setValue(kINTF_TIMER_BEEPS, checkBoxTimerBeeps->isChecked());
+        if (qs.value(kINTF_TIMER_AUTO_START, false).toBool() != checkBoxTimerAutoStart->isChecked())
+        {
+            // We need to update the "rack <--> timer" association
+            shouldEmitUpdate = true;
+            qs.setValue(kINTF_TIMER_AUTO_START, checkBoxTimerAutoStart->isChecked());
+        }
+        qs.setValue(kINTF_CHECK_FOR_UPDATES, checkBoxIntfCheckUpdates->isChecked());
 
         // Duplicate settings
         Settings::Instance().setBool("duplicate.reject-invalid",
@@ -256,14 +293,25 @@ void PrefsDialog::updateSettings()
             shouldEmitUpdate = true;
             qs.setValue(kARBIT_LINK_7P1, checkBoxArbitLink7P1->isChecked());
         }
+        Settings::Instance().setBool("arbitration.solo-auto",
+                                     checkBoxArbitSoloAuto->isChecked());
+        Settings::Instance().setInt("arbitration.solo-players",
+                                    spinBoxArbitSoloPlayers->value());
+        Settings::Instance().setInt("arbitration.solo-value",
+                                    spinBoxArbitSoloValue->value());
         Settings::Instance().setInt("arbitration.search-limit",
                                     spinBoxArbitSearchLimit->value());
         Settings::Instance().setInt("arbitration.penalty-value",
                                     spinBoxArbitPenaltyValue->value());
         Settings::Instance().setInt("arbitration.warnings-limit",
                                     spinBoxArbitWarnLimit->value());
-        Settings::Instance().setInt("arbitration.solo-value",
-                                    spinBoxArbitSoloValue->value());
+
+        // Topping settings
+        Settings::Instance().setBool("topping.elapsed-penalty",
+                                     checkBoxToppingElapsedPenalty->isChecked());
+        Settings::Instance().setInt("topping.timeout-penalty",
+                                    spinBoxToppingExpPenalty->value());
+
 
         // Confirmations settings
         qs.setValue(kCONFO_START_GAME, checkBoxConfoStartGame->isChecked());
