@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2010 Olivier Teulire
+ * Copyright (C) 2010-2012 Olivier Teulire
  * Authors: Olivier Teulire <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,18 +37,53 @@
 #endif
 
 // Convert to/from std::wstring
-std::wstring qtw(const QString &q);
+std::wstring wfq(const QString &q);
 QString qfw(const wstring &wstr);
 // Convert to/from local encoding
 #define qfl(s) qfw(convertToWc(s))
-#define qtl(s) convertToMb(qtw(s))
+#define lfq(s) convertToMb(wfq(s))
 // Convert to/from utf-8 char*
-#define qfu(s) QString::fromUtf8(s)
-#define qtu(s) (s).toUtf8().data()
+#define qfu(s) QString::fromUtf8(string(s).c_str())
+#define ufq(s) (s).toUtf8().data()
 // Translation macro to use gettext
-#define _q(s) QString::fromLocal8Bit(_(s))
+#ifdef __APPLE__
+// On MacOSX, we force the encoding to UTF-8, because we have trouble
+// detecting the current encoding properly (see call to
+// bind_textdomain_codeset() in main.cpp)
+#    define _q(s) qfu(_(s))
+#else
+#    define _q(s) QString::fromLocal8Bit(_(s))
+#endif
 
-// Used for QSettings
-#define ORGANIZATION "eliot"
+
+class QObject;
+class QWidget;
+
+class QtCommon
+{
+public:
+    /// Unit test for the macros above
+    static void CheckConversions();
+
+    /**
+     * Kill the given widget, i.e. hide it, cut its signals, and mark it for deletion.
+     * If a iSource object is given, signals from the source will be disconnected
+     * from the widget to destroy as well.
+     */
+    static void DestroyObject(QWidget *ioWidgetToDestroy,
+                              QObject *iSource = NULL);
+
+    /**
+     * Ask for a confirmation.
+     * Return true if the user confirms, false otherwise.
+     * By default, the question is "Do you want to continue?".
+     * If a parent is provided, the window is centered on it.
+     */
+    static bool requestConfirmation(QString confoKey,
+                                    QString iMsg,
+                                    QString iQuestion = "",
+                                    QWidget *iParent = 0);
+
+};
 
 #endif
