@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,9 @@
 using namespace std;
 
 
+INIT_LOGGER(qt, ScoreWidget);
+
+
 ScoreWidget::ScoreWidget(QWidget *parent, const PublicGame *iGame)
     : QTreeView(parent), m_game(iGame)
 {
@@ -77,7 +80,7 @@ void ScoreWidget::updateModel()
         int rowNum = m_model->rowCount();
         m_model->insertRow(rowNum);
         m_model->setData(m_model->index(rowNum, 0), qfw(p.getName()));
-        m_model->setData(m_model->index(rowNum, 1), p.getPoints());
+        m_model->setData(m_model->index(rowNum, 1), p.getTotalScore());
     }
     resizeColumnToContents(0);
     //resizeColumnToContents(1);
