@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008 Olivier Teulière
+ * Copyright (C) 2008-2012 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,8 @@
 
 #include <QtGui/QTreeView>
 
+#include "logging.h"
+
 
 class PublicGame;
 class QStandardItemModel;
@@ -30,6 +32,7 @@ class QStandardItemModel;
 class ScoreWidget: public QTreeView
 {
     Q_OBJECT;
+    DEFINE_LOGGER();
 
 public:
     explicit ScoreWidget(QWidget *parent = 0, const PublicGame *iGame = NULL);
