@@ -102,10 +102,7 @@ wchar_t *rl_gets()
     wline_read = new wchar_t[len + 1];
     mbstowcs(wline_read, line_read, len + 1);
 
-    if (line_read)
-    {
-        free(line_read);
-    }
+    free(line_read);
 #else
     if (!cin.good())
         return NULL;
@@ -369,6 +366,29 @@ void helpArbitration()
 }
 
 
+void helpTopping()
+{
+    printf("  ?    : aide -- cette page\n");
+    printf("  a [g|gm|gd|l|p|s|S|t|T] : afficher :\n");
+    printf("            g -- grille\n");
+    printf("            gm -- grille + valeur des cases\n");
+    printf("            gd -- grille + debug cross (debug only)\n");
+    printf("            l -- lettres non jouées\n");
+    printf("            p -- partie\n");
+    printf("            r -- recherche\n");
+    printf("            s -- score\n");
+    printf("            S -- score de tous les joueurs\n");
+    printf("            t -- tirage\n");
+    printf("            T -- tirage de tous les joueurs\n");
+    printf("  d [] : vérifier le mot []\n");
+    printf("  j [] {} <> : jouer le mot [] aux coordonnées {} après <> secondes\n");
+    printf("  t [] : simuler un timeout après [] secondes\n");
+    printf("  s [] : sauver la partie en cours dans le fichier []\n");
+    printf("  h [p|n|f|l|r] : naviguer dans l'historique (prev, next, first, last, replay)\n");
+    printf("  q    : quitter le mode topping\n");
+}
+
+
 void help()
 {
     printf("  ?        : aide -- cette page\n");
@@ -376,6 +396,10 @@ void help()
     printf("  ej       : démarrer le mode entraînement en partie joker\n");
     printf("  ee       : démarrer le mode entraînement en partie détonante\n");
     printf("  e8       : démarrer le mode entraînement en partie 7 sur 8\n");
+    printf("  t        : démarrer le mode topping\n");
+    printf("  tj       : démarrer le mode topping en partie joker\n");
+    printf("  te       : démarrer le mode topping en partie détonante\n");
+    printf("  t8       : démarrer le mode topping en partie 7 sur 8\n");
     printf("  d [] {}  : démarrer une partie duplicate avec\n");
     printf("                [] joueurs humains et {} joueurs IA\n");
     printf("  dj [] {} : démarrer une partie duplicate avec\n");
@@ -441,7 +465,7 @@ void displayData(const PublicGame &iGame, const vector<wstring> &tokens)
     else if (displayType == L"S")
         GameIO::printAllPoints(cout, iGame);
     else if (displayType == L"t")
-        GameIO::printPlayedRack(cout, iGame, iGame.getHistory().getSize());
+        GameIO::printPlayedRack(cout, iGame);
     else if (displayType == L"T")
         GameIO::printAllRacks(cout, iGame);
     else
@@ -802,6 +826,48 @@ void loopArbitration(PublicGame &iGame)
 }
 
 
+void loopTopping(PublicGame &iGame)
+{
+    printf("mode topping\n");
+    printf("[?] pour l'aide\n");
+
+    bool quit = false;
+    while (!quit)
+    {
+        const vector<wstring> &tokens = readTokens();
+        if (tokens.empty())
+            continue;
+        try
+        {
+            wchar_t command = parseCharInList(tokens, 0, L"#?adhjstq");
+            if (command == L'?')
+                helpTopping();
+            else if (command == L'q')
+                quit = true;
+            else if (command == L't')
+            {
+                int timeout = parseNum(tokens, 1);
+                iGame.toppingTimeOut(timeout);
+            }
+            else if (command == L'j')
+            {
+                const wstring &word = parseLetters(tokens, 1, iGame.getDic());
+                const wstring &coord = parseAlphaNum(tokens, 2);
+                int elapsed = parseNum(tokens, 3);
+                iGame.toppingPlay(word, coord, elapsed);
+            }
+            else
+                commonCommands(iGame, tokens);
+        }
+        catch (std::exception &e)
+        {
+            printf("%s\n", e.what());
+        }
+    }
+    printf("fin du mode topping\n");
+}
+
+
 void mainLoop(const Dictionary &iDic)
 {
     printf("[?] pour l'aide\n");
@@ -842,7 +908,8 @@ void mainLoop(const Dictionary &iDic)
                                 loopDuplicate(*game);
                             else if (game->getMode() == PublicGame::kARBITRATION)
                                 loopArbitration(*game);
-                            //GameFactory::Instance()->releaseGame(*game);
+                            else if (game->getMode() == PublicGame::kTOPPING)
+                                loopTopping(*game);
                             delete game;
                         }
                         catch (const std::exception &e)
@@ -858,7 +925,6 @@ void mainLoop(const Dictionary &iDic)
                         game->addPlayer(new HumanPlayer);
                         game->start();
                         loopTraining(*game);
-                        //GameFactory::Instance()->releaseGame(*game);
                         delete game;
                     }
                     break;
@@ -874,7 +940,6 @@ void mainLoop(const Dictionary &iDic)
                             game->addPlayer(new AIPercent(1));
                         game->start();
                         loopDuplicate(*game);
-                        //GameFactory::Instance()->releaseGame(*game);
                         delete game;
                     }
                     break;
@@ -890,7 +955,6 @@ void mainLoop(const Dictionary &iDic)
                             game->addPlayer(new AIPercent(1));
                         game->start();
                         loopFreegame(*game);
-                        //GameFactory::Instance()->releaseGame(*game);
                         delete game;
                     }
                     break;
@@ -906,7 +970,16 @@ void mainLoop(const Dictionary &iDic)
                             game->addPlayer(new AIPercent(1));
                         game->start();
                         loopArbitration(*game);
-                        //GameFactory::Instance()->releaseGame(*game);
+                        delete game;
+                    }
+                    break;
+                case L't':
+                    {
+                        // New topping game
+                        PublicGame *game = readGame(iDic, GameParams::kTOPPING, tokens[0]);
+                        game->addPlayer(new HumanPlayer);
+                        game->start();
+                        loopTopping(*game);
                         delete game;
                     }
                     break;
@@ -940,7 +1013,7 @@ int main(int argc, char *argv[])
 
     if (argc != 2 && argc != 3)
     {
-        fprintf(stdout, "Usage: eliot /chemin/vers/ods5.dawg [random_seed]\n");
+        fprintf(stdout, "Usage: eliot /path/to/ods5.dawg [random_seed]\n");
         exit(1);
     }
 
